@@ -9,7 +9,7 @@
 #ifndef ABSYN_H
 #define ABSYN_H
 
-#include "memory.h"
+#include "common.h"
 #include "errormsg.h" // For position
 
 // Forward declaration for markPos.
