@@ -84,14 +84,14 @@ void sumArray(vm::stack *s)
   s->push(sum);
 }
 
-extern const char *emptyarray;
+extern const char *arrayempty;
   
 template<class T, template <class S> class op>
 void binopArray(vm::stack *s)
 {
   array *a=pop<array*>(s);
   size_t size=checkArray(a);
-  if(size == 0) vm::error(emptyarray);
+  if(size == 0) vm::error(arrayempty);
   T m=read<T>(a,0);
   for(size_t i=1; i < size; i++)
     m=op<T>()(m,read<T>(a,i));
@@ -117,7 +117,7 @@ void binopArray2(vm::stack *s)
 	m=op<T>()(m,read<T>(ai,j));
     }
   }
-  if(empty) vm::error(emptyarray);
+  if(empty) vm::error(arrayempty);
   s->push(m);
 }
 
@@ -145,7 +145,7 @@ void binopArray3(vm::stack *s)
       }
     }
   }
-  if(empty) vm::error(emptyarray);
+  if(empty) vm::error(arrayempty);
   s->push(m);
 }
 
@@ -217,7 +217,7 @@ void searchArray(vm::stack *s)
   s->push(0);
 }
 
-extern mem::string emptystring;
+extern string emptystring;
   
 void writestring(vm::stack *s);
   
@@ -227,7 +227,7 @@ void write(vm::stack *s)
   array *a=pop<array*>(s);
   vm::callable *suffix=pop<vm::callable *>(s,NULL);
   T first=pop<T>(s);
-  mem::string S=pop<mem::string>(s,emptystring);
+  string S=pop<string>(s,emptystring);
   vm::item it=pop(s);
   bool defaultfile=isdefault(it);
   camp::file *f=defaultfile ? &camp::Stdout : vm::get<camp::file*>(it);
@@ -258,7 +258,7 @@ void writeArray(vm::stack *s)
 {
   array *A=pop<array*>(s);
   array *a=pop<array*>(s);
-  mem::string S=pop<mem::string>(s,emptystring);
+  string S=pop<string>(s,emptystring);
   vm::item it=pop(s);
   bool defaultfile=isdefault(it);
   camp::file *f=defaultfile ? &camp::Stdout : vm::get<camp::file*>(it);
@@ -276,7 +276,7 @@ void writeArray(vm::stack *s)
       cont=false;
       bool first=true;
       if(i < asize) {
-	if(defaultfile) std::cout << i << ":\t";
+	if(defaultfile) cout << i << ":\t";
 	f->write(read<T>(a,i)); cont=true;
 	first=false;
       }
@@ -286,7 +286,7 @@ void writeArray(vm::stack *s)
 	size_t Ajsize=checkArray(Aj);
 	if(i < Ajsize) {
 	  if(f->text()) {
-	    if(first && defaultfile) std::cout << i << ":\t";
+	    if(first && defaultfile) cout << i << ":\t";
 	    for(unsigned k=0; k <= count; ++k)
 	      f->write(tab);
 	    count=0;
