@@ -43,7 +43,7 @@ using run::less;
 using run::greater;
 using run::plus;
 using run::minus;
-using mem::string;
+
 
 using namespace run;  
   
@@ -51,28 +51,15 @@ void gen_base_venv(venv &ve);
 
 void addType(tenv &te, const char *name, ty *t)
 {
-  te.enter(symbol::trans(name), new tyEntry(t,0,0));
+  te.enter(symbol::trans(name), new tyEntry(t,0,0,position()));
 }
 
 // The base environments for built-in types and functions
 void base_tenv(tenv &te)
 {
-  addType(te, "void", primVoid());
-  addType(te, "bool", primBoolean());
-  addType(te, "int", primInt());
-  addType(te, "real", primReal());
-  addType(te, "string", primString());
-  
-  addType(te, "pair", primPair());
-  addType(te, "triple", primTriple());
-  addType(te, "transform", primTransform());
-  addType(te, "guide", primGuide());
-  addType(te, "path", primPath());
-  addType(te, "pen", primPen());
-  addType(te, "frame", primPicture());
-
-  addType(te, "file", primFile());
-  addType(te, "code", primCode());
+#define PRIMITIVE(name,Name,asyName)  addType(te, #asyName, prim##Name());
+#include <primitives.h>
+#undef PRIMITIVE
 }
 
 const formal noformal(0);  
@@ -96,7 +83,7 @@ void addFunc(venv &ve, access *a, ty *result, symbol *id,
 
   // NOTE: If the function is a field, we should encode the defining record in
   // the entry
-  varEntry *ent = new varEntry(fun, a, 0);
+  varEntry *ent = new varEntry(fun, a, 0, position());
   
   ve.enter(id, ent);
 }
@@ -136,7 +123,7 @@ void addRestFunc(venv &ve, bltin f, ty *result, const char *name, formal frest,
 
   if (frest.t) fun->addRest(frest);
 
-  varEntry *ent = new varEntry(fun, a, 0);
+  varEntry *ent = new varEntry(fun, a, 0, position());
 
   ve.enter(symbol::trans(name), ent);
 }
@@ -340,7 +327,7 @@ void addConstant(venv &ve, T value, ty *t, const char *name,
   item* ref=new item;
   *ref=value;
   access *a = new itemRefAccess(ref);
-  varEntry *ent = new varEntry(t, a, RESTRICTED, module, 0);
+  varEntry *ent = new varEntry(t, a, RESTRICTED, module, 0, position());
   ve.enter(symbol::trans(name), ent);
 }
 
@@ -618,8 +605,8 @@ void addOperators(venv &ve)
 {
   addSimpleOperator(ve,binaryOp<string,plus>,primString(),"+");
   
-  addBooleanOps<bool,And>(ve,primBoolean(),"&&",boolArray());
-  addBooleanOps<bool,Or>(ve,primBoolean(),"||",boolArray());
+  addBooleanOps<bool,And>(ve,primBoolean(),"&",boolArray());
+  addBooleanOps<bool,Or>(ve,primBoolean(),"|",boolArray());
   addBooleanOps<bool,Xor>(ve,primBoolean(),"^",boolArray());
   
   addUnorderedOps<bool>(ve,primBoolean(),boolArray(),boolArray2(),
