@@ -1,7 +1,9 @@
-/* A Bison parser, made by GNU Bison 2.0.  */
+/* A Bison parser, made by GNU Bison 2.3.  */
 
-/* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+/* Skeleton interface for Bison's Yacc-like parsers in C
+
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,13 +17,21 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
-/* As a special exception, when this file is copied by Bison into a
-   Bison output file, you may use that output file without restriction.
-   This special exception was added by the Free Software Foundation
-   in version 1.24 of Bison.  */
+/* As a special exception, you may create a larger work that contains
+   part or all of the Bison parser skeleton and distribute that work
+   under terms of your choice, so long as that work isn't itself a
+   parser generator using the skeleton or a modified version thereof
+   as a parser skeleton.  Alternatively, if you modify or redistribute
+   the parser skeleton itself, you may (at your option) remove this
+   special exception, which will cause the skeleton and the resulting
+   Bison output files to be licensed under the GNU General Public
+   License without this special exception.
+
+   This special exception was added by the Free Software Foundation in
+   version 2.2 of Bison.  */
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -46,17 +56,17 @@
      TENSION = 272,
      ATLEAST = 273,
      CURL = 274,
-     CYCLE = 275,
-     COR = 276,
-     CAND = 277,
-     EQ = 278,
-     NEQ = 279,
-     LT = 280,
-     LE = 281,
-     GT = 282,
-     GE = 283,
-     CARETS = 284,
-     AMPERSAND = 285,
+     COR = 275,
+     CAND = 276,
+     BAR = 277,
+     AMPERSAND = 278,
+     EQ = 279,
+     NEQ = 280,
+     LT = 281,
+     LE = 282,
+     GT = 283,
+     GE = 284,
+     CARETS = 285,
      LOGNOT = 286,
      OPERATOR = 287,
      STRING = 288,
@@ -86,12 +96,14 @@
      STATIC = 312,
      THIS = 313,
      EXPLICIT = 314,
-     LIT = 315,
-     PERM = 316,
-     MODIFIER = 317,
-     UNARY = 318
+     GARBAGE = 315,
+     LIT = 316,
+     PERM = 317,
+     MODIFIER = 318,
+     UNARY = 319
    };
 #endif
+/* Tokens.  */
 #define ID 258
 #define OP 259
 #define ADD 260
@@ -109,17 +121,17 @@
 #define TENSION 272
 #define ATLEAST 273
 #define CURL 274
-#define CYCLE 275
-#define COR 276
-#define CAND 277
-#define EQ 278
-#define NEQ 279
-#define LT 280
-#define LE 281
-#define GT 282
-#define GE 283
-#define CARETS 284
-#define AMPERSAND 285
+#define COR 275
+#define CAND 276
+#define BAR 277
+#define AMPERSAND 278
+#define EQ 279
+#define NEQ 280
+#define LT 281
+#define LE 282
+#define GT 283
+#define GE 284
+#define CARETS 285
 #define LOGNOT 286
 #define OPERATOR 287
 #define STRING 288
@@ -149,17 +161,19 @@
 #define STATIC 312
 #define THIS 313
 #define EXPLICIT 314
-#define LIT 315
-#define PERM 316
-#define MODIFIER 317
-#define UNARY 318
+#define GARBAGE 315
+#define LIT 316
+#define PERM 317
+#define MODIFIER 318
+#define UNARY 319
 
 
 
 
-#if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 41 "camp.y"
-typedef union YYSTYPE {
+#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
+typedef union YYSTYPE
+#line 46 "camp.y"
+{
   position pos;
   //sym::symbol *sym;
   struct {
@@ -202,9 +216,10 @@ typedef union YYSTYPE {
   //absyntax::funheader *fh;
   absyntax::formal *fl;
   absyntax::formals *fls;
-} YYSTYPE;
-/* Line 1318 of yacc.c.  */
-#line 208 "camp.tab.h"
+}
+/* Line 1529 of yacc.c.  */
+#line 222 "camp.tab.h"
+	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -212,5 +227,3 @@ typedef union YYSTYPE {
 
 extern YYSTYPE yylval;
 
-
-
