@@ -34,12 +34,12 @@ void reportWarning(const string& desc)
   em->sync();
 }
 
-void reportError(const std::ostringstream& desc)
+void reportError(const ostringstream& desc)
 {
   reportError(desc.str());
 }
   
-void reportWarning(const std::ostringstream& desc)
+void reportWarning(const ostringstream& desc)
 {
   reportWarning(desc.str());
 }
