@@ -11,6 +11,7 @@
 
 #include <cfloat>
 
+#include "common.h"
 #include "stack.h"
 #include "fileio.h"
 #include "lexical.h"
@@ -21,7 +22,6 @@ namespace run {
 
 using vm::read;
 using vm::pop;
-using mem::string;
 
 template<class T, class S>
 void cast(vm::stack *s)
@@ -39,7 +39,7 @@ void castDoubleInt(vm::stack *s)
 template<class T>
 void stringCast(vm::stack *s)
 {
-  std::ostringstream buf;
+  ostringstream buf;
   buf.precision(DBL_DIG);
   buf << pop<T>(s);
   s->push((string) buf.str());
@@ -75,10 +75,11 @@ template<class T>
 void read(vm::stack *s)
 {
   camp::file *f = pop<camp::file*>(s);
-  T val;
+  T val=T();
   if(f->isOpen()) {
     f->read(val);
     if(f->LineMode()) f->nexteol();
+    if(interact::interactive) f->purgeStandard(val);
   }
   s->push(val);
 }
@@ -86,7 +87,7 @@ void read(vm::stack *s)
 inline int Limit(int nx) {return nx == 0 ? INT_MAX : nx;}
 inline void reportEof(camp::file *f, int count) 
 {
-  std::ostringstream buf;
+  ostringstream buf;
   buf << "EOF after reading " << count
       << " values from file '" << f->filename() << "'.";
   vm::error(buf);
@@ -162,6 +163,7 @@ void readArray(vm::stack *s)
 	if(f->LineMode() && f->nexteol()) break;
       }
     }
+    if(interact::interactive) f->purgeStandard(v);
   }
   s->push(c);
 }
