@@ -11,6 +11,7 @@
 #include "coenv.h"
 #include "dec.h"
 #include "fundec.h"
+#include "newexp.h"
 #include "stm.h"
 #include "exp.h"
 #include "modifier.h"
@@ -21,10 +22,12 @@ namespace absyntax {
 
 using namespace trans;
 using namespace types;
-  
+
+using mem::list;
+
 trans::tyEntry *ty::transAsTyEntry(coenv &e, record *where)
 {
-  return new trans::tyEntry(trans(e, false), 0, where);
+  return new trans::tyEntry(trans(e, false), 0, where, getPos());
 }
 
 
@@ -293,7 +296,8 @@ types::ty *decidstart::getType(types::ty *base, coenv &, bool)
 trans::tyEntry *decidstart::getTyEntry(trans::tyEntry *base, coenv &e,
                                        record *where)
 {
-  return dims ? new trans::tyEntry(getType(base->t,e,false), 0, where) :
+  return dims ? new trans::tyEntry(getType(base->t,e,false), 0,
+                                   where, getPos()) :
                 base;
 }
 
@@ -334,7 +338,7 @@ types::ty *fundecidstart::getType(types::ty *base, coenv &e, bool tacit)
 trans::tyEntry *fundecidstart::getTyEntry(trans::tyEntry *base, coenv &e,
                                           record *where)
 {
-  return new trans::tyEntry(getType(base->t,e,false), 0, where);
+  return new trans::tyEntry(getType(base->t,e,false), 0, where, getPos());
 }
 
 void fundecidstart::addOps(types::ty *base, coenv &e, record *r)
@@ -361,17 +365,17 @@ void decid::prettyprint(ostream &out, int indent)
 
 
 varEntry *makeVarEntryWhere(coenv &e, record *r, types::ty *t,
-                                record *where)
+                            record *where, position pos)
 {
   access *a = r ? r->allocField(e.c.isStatic()) :
                   e.c.allocLocal();
 
-  return r ? new varEntry(t, a, e.c.getPermission(), r, where) :
-             new varEntry(t, a, where);
+  return r ? new varEntry(t, a, e.c.getPermission(), r, where, pos) :
+             new varEntry(t, a, where, pos);
 }
 
-varEntry *makeVarEntry(coenv &e, record *r, types::ty *t) {
-  return makeVarEntryWhere(e, r, t, r);
+varEntry *makeVarEntry(position pos, coenv &e, record *r, types::ty *t) {
+  return makeVarEntryWhere(e, r, t, r, pos);
 }
 
 void addVar(coenv &e, record *r, varEntry *v, symbol *id)
@@ -401,11 +405,19 @@ void initializeVar(position pos, coenv &e, record *,
 void createVar(position pos, coenv &e, record *r,
                symbol *id, types::ty *t, varinit *init)
 {
-  varEntry *v=makeVarEntry(e, r, t);
+  varEntry *v=makeVarEntry(pos, e, r, t);
   addVar(e, r, v, id);
   initializeVar(pos, e, r, v, t, init);
 }
 
+void createVarOutOfOrder(position pos, coenv &e, record *r,
+                         symbol *id, types::ty *t, varinit *init)
+{
+  varEntry *v=makeVarEntry(pos, e, r, t);
+  initializeVar(pos, e, r, v, t, init);
+  addVar(e, r, v, id);
+}
+
 void addTypeWithPermission(coenv &e, record *r, tyEntry *base, symbol *id)
 {
   // Only bother encoding permissions for private types.
@@ -418,13 +430,6 @@ void addTypeWithPermission(coenv &e, record *r, tyEntry *base, symbol *id)
   e.e.addType(id, ent);
 }
 
-void createVarOutOfOrder(position pos, coenv &e, record *r,
-                         symbol *id, types::ty *t, varinit *init)
-{
-  varEntry *v=makeVarEntry(e, r, t);
-  initializeVar(pos, e, r, v, t, init);
-  addVar(e, r, v, id);
-}
 
 void decid::transAsField(coenv &e, record *r, types::ty *base)
 {
@@ -528,7 +533,7 @@ public:
 // the import, but doesn't add the import to the environment.
 varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
 {
-  record *imp=e.e.getModule(id, (mem::string)*id);
+  record *imp=e.e.getModule(id, (string)*id);
   if (!imp) {
     em->error(pos);
     *em << "could not load module '" << *id << "'";
@@ -543,7 +548,7 @@ varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
 
     // The varEntry should have whereDefined()==0 as it is not defined inside
     // the record r.
-    varEntry *v=makeVarEntryWhere(e, r, imp, 0);
+    varEntry *v=makeVarEntryWhere(e, r, imp, 0, pos);
     initializeVar(pos, e, r, v, imp, &init);
     return v;
   }
@@ -581,7 +586,7 @@ void idpair::transAsUnravel(coenv &e, record *r,
 
 void idpairlist::prettyprint(ostream &out, int indent)
 {
-  for (mem::list<idpair *>::iterator p=base.begin();
+  for (list<idpair *>::iterator p=base.begin();
        p != base.end();
        ++p)
     (*p)->prettyprint(out, indent);
@@ -589,7 +594,7 @@ void idpairlist::prettyprint(ostream &out, int indent)
 
 void idpairlist::transAsAccess(coenv &e, record *r)
 {
-  for (mem::list<idpair *>::iterator p=base.begin();
+  for (list<idpair *>::iterator p=base.begin();
        p != base.end();
        ++p)
     (*p)->transAsAccess(e,r);
@@ -598,7 +603,7 @@ void idpairlist::transAsAccess(coenv &e, record *r)
 void idpairlist::transAsUnravel(coenv &e, record *r,
                                 protoenv &source, varEntry *qualifier)
 {
-  for (mem::list<idpair *>::iterator p=base.begin();
+  for (list<idpair *>::iterator p=base.begin();
        p != base.end();
        ++p)
     (*p)->transAsUnravel(e,r,source,qualifier);
@@ -691,10 +696,7 @@ void includedec::loadFailed(coenv &)
 
 void includedec::transAsField(coenv &e, record *r)
 {
-  if(settings::verbose > 1)
-    std::cerr << "Including " <<  filename << std::endl;
-  
-  file *ast = parser::parseFile(filename);
+  file *ast = parser::parseFile(filename,"Including");
   em->sync();
 
   // The runnables will be run, one at a time, without any additional scoping.
@@ -718,12 +720,29 @@ void recorddec::prettyprint(ostream &out, int indent)
   body->prettyprint(out, indent+1);
 }
 
+void recorddec::transRecordInitializer(coenv &e, record *parent)
+{
+  position here=getPos();
+
+  // This is equivalent to the code
+  //   A operator init() { return new A; };
+  // where A is the name of the record.
+  formals formals(here);
+  simpleName recordName(here, id);
+  nameTy result(here, &recordName);
+  newRecordExp exp(here, &result);
+  returnStm stm(here, &exp);
+  fundec init(here, &result, symbol::opTrans("init"), &formals, &stm);
+
+  init.transAsField(e, parent);
+}
+
 void recorddec::transAsField(coenv &e, record *parent)
 {
   record *r = parent ? parent->newRecord(id, e.c.isStatic()) :
                        e.c.newRecord(id);
                      
-  addTypeWithPermission(e, parent, new trans::tyEntry(r,0,parent), id);
+  addTypeWithPermission(e, parent, new trans::tyEntry(r,0,parent,getPos()), id);
   e.e.addRecordOps(r);
   if (parent)
     parent->e.addRecordOps(r);
@@ -733,6 +752,11 @@ void recorddec::transAsField(coenv &e, record *parent)
   coenv re(c,e.e);
   
   body->transAsRecordBody(re, r);
+
+  // After the record is translated, add a default initializer so that a
+  // variable of the type of the record is initialized to a new instance by
+  // default.
+  transRecordInitializer(e, parent);
 }  
 
 runnable *autoplainRunnable() {
