@@ -36,7 +36,6 @@ class lambda;
 }
 namespace absyntax {
 
-using mem::list;
 using trans::genv;
 using trans::coenv;
 using trans::protoenv;
@@ -129,6 +128,13 @@ public:
     transAsField(e, 0);
   }
 
+  /* This can be overridden, to specify a special way of translating the code
+   * when it is run at the top of the interactive prompt.
+   */
+  virtual void interactiveTrans(coenv &e) {
+    trans(e);
+  }
+
   void markTransAsField(coenv &e, record *r)
   {
     markPos(e);
@@ -157,7 +163,7 @@ public:
 
 class block : public runnable {
 public:
-  list<runnable *> stms;
+  mem::list<runnable *> stms;
 
   // If the runnables should be interpreted in their own scope.
   bool scope;
@@ -201,8 +207,8 @@ public:
 };
 
 class modifierList : public absyn {
-  list<trans::permission> perms;
-  list<trans::modifier> mods;
+  mem::list<trans::permission> perms;
+  mem::list<trans::modifier> mods;
 
 public:
   modifierList(position pos)
@@ -325,7 +331,7 @@ public:
 };
 
 class decidlist : public absyn {
-  list<decid *> decs;
+  mem::list<decid *> decs;
 
 public:
   decidlist(position pos)
@@ -514,10 +520,10 @@ public:
 // Parses the file given, and translates the resulting runnables as if they
 // occurred at this place in the code.
 class includedec : public dec {
-  mem::string filename;
+  string filename;
 
 public:
-  includedec(position pos, mem::string filename)
+  includedec(position pos, string filename)
     : dec(pos), filename(filename) {}
   includedec(position pos, symbol *id)
     : dec(pos), filename(*id) {}
@@ -549,6 +555,8 @@ class recorddec : public dec {
   symbol *id;
   block *body;
 
+  void transRecordInitializer(coenv &e, record *parent);
+
 public:
   recorddec(position pos, symbol *id, block *body)
     : dec(pos), id(id), body(body) {}
