@@ -8,18 +8,15 @@
 #ifndef DRAWELEMENT_H
 #define DRAWELEMENT_H
 
-#include <list>
-#include <string>
 #include <vector>
 
+#include "common.h"
 #include "bbox.h"
 #include "pen.h"
 #include "psfile.h"
 #include "texfile.h"
 #include "pipestream.h"
 
-using std::string;
-
 namespace camp {
 
 class box {
@@ -129,7 +126,7 @@ public:
 };
 
 // Base class for drawElements that involve paths.
-class drawPathBase : public drawElement {
+class drawPathBase : public virtual drawElement {
 protected:
   path p;
 
@@ -143,9 +140,18 @@ public:
 
   virtual ~drawPathBase() {}
 
-  void bounds(bbox& b, iopipestream&, boxvector&, bboxlist&) {
+  virtual void bounds(bbox& b, iopipestream&, boxvector&, bboxlist&) {
     b += p.bounds();
   }
+  
+  virtual void writepath(psfile *out) {
+    out->write(p);
+  }
+  
+  virtual void writeshiftedpath(texfile *out) {
+    out->writeshifted(p);
+  }
+  
 };
 
 // Base class for drawElements that involve paths and pens.
@@ -178,18 +184,6 @@ public:
     return p.cyclic();
   }
   
-  virtual void bounds(bbox& b, iopipestream&, boxvector&, bboxlist&) {
-    b += p.bounds();
-  }
-  
-  virtual void writepath(psfile *out) {
-    out->write(p);
-  }
-  
-  virtual void writeshiftedpath(texfile *out) {
-    out->writeshifted(p);
-  }
-  
   virtual void penStart(psfile *out)
   {
     if (t())
@@ -213,6 +207,8 @@ public:
     if (t())
       out->grestore();
   }
+  
+  
 };
   
 // Base class for drawElements that involve superpaths and pens.
