@@ -15,8 +15,8 @@
 
 namespace camp {
   
-class drawLabel : public drawElement {
-private:
+class drawLabel : public virtual drawElement {
+protected:
   string label,size;
   transform T;		// A linear (shiftless) transformation.
   pair position;
@@ -29,38 +29,58 @@ private:
   pair Align;
   pair texAlign;
   bbox Box;
+  bool enabled;
   
 public:
-  drawLabel(string label, string size, transform T, pair position, pair align,
-	   pen pentype)
+  drawLabel(string label, string size, transform T, pair position,
+	    pair align, pen pentype)
     : label(label), size(size), T(shiftless(T)), position(position),
       align(align), pentype(pentype), width(0.0), height(0.0), depth(0.0),
-      havebounds(false), suppress(false) {} 
+    havebounds(false), suppress(false), enabled(false) {} 
   
   virtual ~drawLabel() {}
 
+  void getbounds(iopipestream& tex, const string& texengine);
+  
+  void checkbounds();
+    
   void bounds(bbox& b, iopipestream&, boxvector&, bboxlist&);
   
-  bool texbounds(iopipestream& tex, string& s, bool warn);
+  bool texbounds(iopipestream& tex, string& s, const char **abort, bool warn);
     
+  int wait(iopipestream &tex, const char *s, const char **abort,
+	   bool ignore=false);
+  
   bool islabel() {
     return true;
   }
 
-  bool write(texfile *out) {
-    if(!havebounds) 
-      reportError("drawLabel::write called before bounds");
-    if(suppress || pentype.invisible()) return true;
-    out->setpen(pentype);
-    out->put(label,T,position,texAlign);
-    return true;
-  }
+  bool write(texfile *out);
 
   drawElement *transformed(const transform& t);
   
   void labelwarning(const char *action); 
 };
 
+class drawLabelPath : public drawLabel, public drawPathPenBase {
+private:  
+  string justify;
+  pair shift;
+public:
+  drawLabelPath(string label, string size, path src,
+		string justify, pair shift, pen pentype) : 
+    drawLabel(label,size,identity(),pair(0.0,0.0),align,pentype),
+    drawPathPenBase(src,pentype), justify(justify), shift(shift) {}
+  
+  virtual ~drawLabelPath() {}
+
+  void bounds(bbox& b, iopipestream& tex, boxvector&, bboxlist&);
+  
+  bool write(texfile *out);
+  
+  drawElement *transformed(const transform& t);
+};
+
 }
 
 #endif
