@@ -12,10 +12,7 @@
 
 #include <iostream>
 
-using std::cout;
-using std::endl;
-
-#include "memory.h"
+#include "common.h"
 #include "frame.h"
 #include "table.h"
 #include "types.h"
@@ -62,10 +59,14 @@ class entry : public gc {
   // The record where the variable or type is defined, or 0 if the entry is
   // not a field.
   record *where;
+
+  // The location (file and line number) where the entry was defined.
+  position pos;
   
 public:
-  entry(record *where) : where(where) {}
-  entry(permission perm, record *r, record *where) : where(where) {
+  entry(record *where, position pos) : where(where), pos(pos) {}
+  entry(permission perm, record *r, record *where, position pos)
+    : where(where), pos(pos) {
     addPerm(perm, r);
   }
 
@@ -82,6 +83,10 @@ public:
   record *whereDefined() {
     return where;
   }
+  
+  position getPos() {
+    return pos;
+  }
 };
     
 class varEntry : public entry {
@@ -89,11 +94,12 @@ class varEntry : public entry {
   access *location;
 
 public:
-  varEntry(ty *t, access *location, record *where)
-    : entry(where), t(t), location(location) {}
+  varEntry(ty *t, access *location, record *where, position pos)
+    : entry(where, pos), t(t), location(location) {}
 
-  varEntry(ty *t, access *location, permission perm, record *r, record *where)
-    : entry(perm, r, where), t(t), location(location) {}
+  varEntry(ty *t, access *location, permission perm, record *r,
+           record *where, position pos)
+    : entry(perm, r, where, pos), t(t), location(location) {}
 
   // (Non-destructively) merges two varEntries, created a qualified varEntry.
   varEntry(varEntry &qv, varEntry &v);
@@ -126,8 +132,8 @@ public:
   ty *t;
   varEntry *v;  // NOTE: Name isn't very descriptive.
 
-  tyEntry(ty *t, varEntry *v, record *where)
-    : entry(where), t(t), v(v) {}
+  tyEntry(ty *t, varEntry *v, record *where, position pos)
+    : entry(where, pos), t(t), v(v) {}
 
   tyEntry(tyEntry *base, permission perm, record *r)
     : entry(*base, perm, r), t(base->t), v(base->v) {}
@@ -320,12 +326,23 @@ public:
     }
     scopes.pop();
   }
+  
+  // Adds the definitions of the top-level scope to the level underneath,
+  // and then removes the top scope.
+  void collapseScope() {
+    // NOTE: May be expensively copying a large hash table.
+    keymultimap top=scopes.top();
+    scopes.pop();
+
+    keymultimap& underneath=scopes.top();
+    underneath.insert(top.begin(), top.end());
+  }
 
   // Prints a list of the variables to the standard output.
   void list(record *module=0);
 
   // Adds to l, all names prefixed by start.
-  void completions(mem::list<symbol *>& l, mem::string start);
+  void completions(mem::list<symbol *>& l, string start);
 };
 #endif
 
