@@ -13,6 +13,7 @@
 
 using namespace types;
 
+
 namespace trans {
 
 // Instances of this class are passed to types::ty objects so that they can call
@@ -142,7 +143,7 @@ env::env(genv &ge)
   base_venv(ve);
 }
 
-record *env::getModule(symbol *id, mem::string filename)
+record *env::getModule(symbol *id, string filename)
 {
   return ge.getModule(id, filename);
 }
