@@ -82,9 +82,9 @@ types::ty *tempExp::trans(coenv &e) {
 
 
 varEntryExp::varEntryExp(position pos, types::ty *t, access *a)
-  : exp(pos), v(new trans::varEntry(t, a, 0)) {}
+  : exp(pos), v(new trans::varEntry(t, a, 0, position())) {}
 varEntryExp::varEntryExp(position pos, types::ty *t, vm::bltin f)
-  : exp(pos), v(new trans::varEntry(t, new bltinAccess(f), 0)) {}
+  : exp(pos), v(new trans::varEntry(t, new bltinAccess(f), 0, position())) {}
 
 types::ty *varEntryExp::getType(coenv &) {
   return v->getType();
@@ -269,6 +269,8 @@ types::ty *thisExp::getType(coenv &e)
 
 void scaleExp::prettyprint(ostream &out, int indent)
 {
+  exp *left=getLeft(); exp *right=getRight();
+
   prettyname(out, "scaleExp",indent);
   left->prettyprint(out, indent+1);
   right->prettyprint(out, indent+1);
@@ -276,6 +278,8 @@ void scaleExp::prettyprint(ostream &out, int indent)
 
 types::ty *scaleExp::trans(coenv &e)
 {
+  exp *left=getLeft(); exp *right=getRight();
+
   types::ty *lt = left->cgetType(e);
   if (lt->kind != types::ty_int && lt->kind != types::ty_real) {
     if (lt->kind != types::ty_error) {
@@ -331,7 +335,7 @@ void stringExp::prettyprint(ostream &out, int indent)
 
 types::ty *stringExp::trans(coenv &e)
 {
-  e.c.encode(inst::constpush,(item) mem::string(str));
+  e.c.encode(inst::constpush,(item) string(str));
   
   return types::primString();  
 }
@@ -352,7 +356,7 @@ types::ty *booleanExp::trans(coenv &e)
 
 void newPictureExp::prettyprint(ostream &out, int indent)
 {
-  prettyname(out, "newFrameExp",indent);
+  prettyname(out, "newPictureExp",indent);
 }
 
 types::ty *newPictureExp::trans(coenv &e)
@@ -362,6 +366,18 @@ types::ty *newPictureExp::trans(coenv &e)
   return types::primPicture();  
 }
 
+void cycleExp::prettyprint(ostream &out, int indent)
+{
+  prettyname(out, "cycleExp",indent);
+}
+
+types::ty *cycleExp::trans(coenv &e)
+{
+  e.c.encode(inst::builtin, run::newCycleToken);
+  
+  return types::primCycleToken();  
+}
+
 void nullPathExp::prettyprint(ostream &out, int indent)
 {
   prettyname(out, "nullPathExp",indent);
@@ -594,6 +610,10 @@ types::ty *callExp::trans(coenv &e)
 
   application *a= ca ? ca : getApplication(e);
   
+  // The cached application is no longer needed after translation, so let be
+  // garbage collected.
+  ca=0;
+
   if (!a)
     return primError();
 
@@ -767,6 +787,23 @@ void conditionalExp::prettyprint(ostream &out, int indent)
   onFalse->prettyprint(out, indent+1);
 }
 
+void conditionalExp::baseTransToType(coenv &e, types::ty *target) {
+  test->transToType(e, types::primBoolean());
+
+  int tlabel = e.c.fwdLabel();
+  e.c.useLabel(inst::cjmp,tlabel);
+
+  onFalse->transToType(e, target);
+
+  int end = e.c.fwdLabel();
+  e.c.useLabel(inst::jmp,end);
+
+  e.c.defLabel(tlabel);
+  onTrue->transToType(e, target);
+
+  e.c.defLabel(end);
+}
+
 void conditionalExp::transToType(coenv &e, types::ty *target)
 {
   if (isAnArray(e, test)) {
@@ -780,20 +817,7 @@ void conditionalExp::transToType(coenv &e, types::ty *target)
     e.c.encode(inst::builtin, run::arrayConditional);
   }
   else {
-    test->transToType(e, types::primBoolean());
-
-    int tlabel = e.c.fwdLabel();
-    e.c.useLabel(inst::cjmp,tlabel);
-
-    onFalse->transToType(e, target);
-
-    int end = e.c.fwdLabel();
-    e.c.useLabel(inst::jmp,end);
-
-    e.c.defLabel(tlabel);
-    onTrue->transToType(e, target);
-
-    e.c.defLabel(end);
+    baseTransToType(e, target);
   }
 }
 
@@ -868,26 +892,6 @@ types::ty *conditionalExp::getType(coenv &e)
 }
  
 
-types::ty *andOrExp::trans(coenv &e)
-{
-  if (isAnArray(e,left) || isAnArray(e,right)) {
-    binaryExp be(getPos(), left, op, right);
-    return be.trans(e);
-  }
-  else
-    return baseTrans(e);
-}
-
-types::ty *andOrExp::getType(coenv &e)
-{
-  if (isAnArray(e,left) || isAnArray(e,right)) {
-    binaryExp be(getPos(), left, op, right);
-    return be.cgetType(e);
-  }
-  else
-    return baseGetType(e);
-}
-
 void orExp::prettyprint(ostream &out, int indent)
 {
   prettyname(out, "orExp", indent);
@@ -896,16 +900,16 @@ void orExp::prettyprint(ostream &out, int indent)
   right->prettyprint(out, indent+1);
 }
 
-types::ty *orExp::baseTrans(coenv &e)
+types::ty *orExp::trans(coenv &e)
 {
   //     a || b
   // translates into
   //     a ? true : b
   booleanExp be(pos, true);
   conditionalExp ce(pos, left, &be, right);
-  ce.transToType(e, primBoolean());
+  ce.baseTransToType(e, primBoolean());
 
-  return baseGetType(e);
+  return getType(e);
 }
 
 
@@ -917,16 +921,16 @@ void andExp::prettyprint(ostream &out, int indent)
   right->prettyprint(out, indent+1);
 }
 
-types::ty *andExp::baseTrans(coenv &e)
+types::ty *andExp::trans(coenv &e)
 {
   //     a && b
   // translates into
   //     a ? b : false
   booleanExp be(pos, false);
   conditionalExp ce(pos, left, right, &be);
-  ce.transToType(e, primBoolean());
+  ce.baseTransToType(e, primBoolean());
 
-  return cgetType(e);
+  return getType(e);
 }
 
 
