@@ -24,7 +24,6 @@ class application;
 
 namespace absyntax {
 
-using mem::list;
 using trans::coenv;
 using trans::application;
 using trans::access;
@@ -55,6 +54,14 @@ public:
   // expression.  ie. 3sin(x)
   virtual bool scalable() { return true; }
   
+  // Specifies if the value of the expression should be written to interactive
+  // prompt if typed as a stand-alone expression.  For example:
+  // > 2+3;
+  // should write 5, but
+  // > x=2+3;
+  // shouldn't.  (This choices are largely aesthetic)
+  virtual bool writtenToPrompt() { return true; }
+
   // Translates the expression to the given target type.  This should only be
   // called with a type returned by getType().  It does not perform implicit
   // casting.
@@ -187,11 +194,17 @@ public:
 
   void transAsType(coenv &e, types::ty *target) {
     value->varTrans(trans::READ, e, target);
+    
+    // After translation, the cached type is no longer needed and should be
+    // garbage collected.  This could presumably be done in every class derived
+    // from exp, but here it is most important as nameExp can have heavily
+    // overloaded types cached.
+    ct=0;
   }
 
   types::ty *trans(coenv &e) {
-    types::ty *t = value->varGetType(e);
-    if (t == 0) {
+    types::ty *t=cgetType(e);
+    if (t->kind == types::ty_error) {
       em->error(getPos());
       *em << "no matching variable \'" << *value << "\'";
       return types::primError();
@@ -214,10 +227,14 @@ public:
 
   void transWrite(coenv &e, types::ty *target) {
     value->varTrans(trans::WRITE, e, target);
+
+    ct=0;  // See note in transAsType.
   }
   
   void transCall(coenv &e, types::ty *target) {
     value->varTrans(trans::CALL, e, target);
+
+    ct=0;  // See note in transAsType.
   }
 
   exp *evaluate(coenv &, types::ty *) {
@@ -383,7 +400,7 @@ public:
 
 
 class stringExp : public literalExp {
-  mem::string str;
+  string str;
 
 public:
   stringExp(position pos, string str)
@@ -408,6 +425,18 @@ public:
   types::ty *getType(coenv &) { return types::primBoolean(); }
 };
 
+class cycleExp : public literalExp {
+
+public:
+  cycleExp(position pos)
+    : literalExp(pos) {}
+
+  void prettyprint(ostream &out, int indent);
+
+  types::ty *trans(coenv &e);
+  types::ty *getType(coenv &) { return types::primCycleToken(); }
+};
+
 class newPictureExp : public literalExp {
 
 public:
@@ -677,13 +706,15 @@ public:
 
 // Scaling expressions such as 3sin(x).
 class scaleExp : public binaryExp {
-  exp *left;
-  exp *right;
-
+  exp *getLeft() {
+    return (*this->args)[0].val;
+  }
+  exp *getRight() {
+    return (*this->args)[1].val;
+  }
 public:
   scaleExp(position pos, exp *left, exp *right)
-    : binaryExp(pos, left, symbol::trans("*"), right), left(left),
-      right(right) {}
+    : binaryExp(pos, left, symbol::trans("*"), right) {}
 
   void prettyprint(ostream &out, int indent);
 
@@ -713,6 +744,8 @@ public:
 
   void prettyprint(ostream &out, int indent);
 
+  void baseTransToType(coenv &e, types::ty *target);
+
   void transToType(coenv &e, types::ty *target);
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
@@ -729,11 +762,8 @@ public:
   andOrExp(position pos, exp *left, symbol *op, exp *right)
     : exp(pos), left(left), op(op), right(right) {}
 
-  types::ty *trans(coenv &e);
-  types::ty *getType(coenv &);
-
-  virtual types::ty *baseTrans(coenv &e) = 0;
-  virtual types::ty *baseGetType(coenv &) {
+  virtual types::ty *trans(coenv &e) = 0;
+  virtual types::ty *getType(coenv &) {
     return types::primBoolean();
   }
 };
@@ -745,7 +775,7 @@ public:
 
   void prettyprint(ostream &out, int indent);
 
-  types::ty *baseTrans(coenv &e);
+  types::ty *trans(coenv &e);
 };
 
 class andExp : public andOrExp {
@@ -755,7 +785,7 @@ public:
 
   void prettyprint(ostream &out, int indent);
 
-  types::ty *baseTrans(coenv &e);
+  types::ty *trans(coenv &e);
 };
 
 class joinExp : public callExp {
@@ -810,6 +840,9 @@ public:
 
   void prettyprint(ostream &out, int indent);
 
+  // Don't write the result of an assignment to the prompt.
+  bool writtenToPrompt() { return false; }
+
   void transAsType(coenv &e, types::ty *target);
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
@@ -841,6 +874,9 @@ public:
 
   bool scalable() { return false; }
 
+  // Don't write the result to the prompt.
+  bool writtenToPrompt() { return false; }
+
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
 };
