@@ -36,6 +36,7 @@ using namespace types;
 using settings::getSetting;
 using settings::Setting;
 
+
 namespace trans {
 
 genv::genv()
@@ -44,9 +45,6 @@ genv::genv()
   // Add settings as a module.  This is so that the init file ~/.asy/config.asy
   // can set settings.
   imap["settings"]=settings::getSettingsModule();
-#ifdef HAVE_LIBGSL  
-  imap["gsl"]=trans::getGSLModule();
-#endif  
 
   // Translate plain in advance, if we're using autoplain.
   if(getSetting<bool>("autoplain")) {
@@ -57,14 +55,14 @@ genv::genv()
 
     Setting("autoplain")=true;
   }
+#ifdef HAVE_LIBGSL  
+  imap["gsl"]=trans::getGSLModule();
+#endif  
 }
 
-record *genv::loadModule(symbol *id, mem::string filename) {
-  if(settings::verbose > 1)
-    std::cerr << "Loading " <<  filename << std::endl;
-    
+record *genv::loadModule(symbol *id, string filename) {
   // Get the abstract syntax tree.
-  absyntax::file *ast = parser::parseFile(filename);
+  absyntax::file *ast = parser::parseFile(filename,"Loading");
   
   inTranslation.push_front(filename);
 
@@ -77,7 +75,7 @@ record *genv::loadModule(symbol *id, mem::string filename) {
   return r;
 }
 
-void genv::checkRecursion(mem::string filename) {
+void genv::checkRecursion(string filename) {
   if (find(inTranslation.begin(), inTranslation.end(), filename) !=
          inTranslation.end()) {
     em->sync();
@@ -87,7 +85,7 @@ void genv::checkRecursion(mem::string filename) {
   }
 }
 
-record *genv::getModule(symbol *id, mem::string filename) {
+record *genv::getModule(symbol *id, string filename) {
   checkRecursion(filename);
 
   record *r=imap[filename];
@@ -113,7 +111,7 @@ importInitMap *genv::getInitMap()
     genv &ge;
     initMap(genv &ge)
       : ge(ge) {}
-    lambda *operator[](mem::string s) {
+    lambda *operator[](string s) {
       record *r=ge.imap[s];
       return r ? r->getInit() : 0;
     }
