@@ -21,7 +21,6 @@
 #include "util.h"
 #include "errormsg.h"
 
-using std::cout;
 using namespace settings;
 
 namespace run {
@@ -81,32 +80,27 @@ void init_interactive()
   
 #if !defined(HAVE_LIBREADLINE) || !defined(HAVE_LIBCURSES)
 char *readline(const char *prompt) {
-  std::cout << prompt;
+  if(!cin.good()) {cin.clear(); return NULL;}
+  cout << prompt;
   string s;
-  getline(std::cin,s);
+  getline(cin,s);
   char *p=(char *) malloc(s.size()+1);
   return strcpy(p,s.c_str());
 }
 #endif  
   
-mem::string simpleline() {
+string simpleline(string prompt) {
   // Rebind tab key, as the setting tabcompletion may be changed at runtime.
   pre_readline();
 
   /* Get a line from the user. */
-  char *line = readline(getSetting<mem::string>("prompt").c_str());
+  char *line = readline(prompt.c_str());
 
   /* Ignore keyboard interrupts while taking input. */
   errorstream::interrupt=false;
 
   if (line) {
-    /* If the line has any text in it, save it on the history. */
-#if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
-    if (line[0] != '\0' && line[0] != '\n')
-      add_history(line);
-#endif    
-
-    mem::string s=line;
+    string s=line;
     free(line);
     return s;
   } else {
@@ -115,6 +109,47 @@ mem::string simpleline() {
   }
 }
 
+void addToHistory(string line) {
+#if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
+    // Only add it if it has something other than newlines.
+    if (line.find_first_not_of('\n') != string::npos) {
+      add_history(line.c_str());
+    }
+#endif    
+}
+
+string getLastHistoryLine() {
+#if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
+  HIST_ENTRY *entry=history_list()[history_length-1];
+  if (!entry) {
+    em->compiler(position());
+    *em << "can't access last history line";
+    return "";
+  }
+  else {
+    return entry->line;
+  }
+#else
+  return "";
+#endif
+}
+
+void setLastHistoryLine(string line) {
+#if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
+  HIST_ENTRY *entry=remove_history(history_length-1);
+  if (!entry) {
+    em->compiler(position());
+    *em << "can't modify last history line";
+  }
+  else {
+    addToHistory(line);
+
+    free(entry->line);
+    free(entry);
+  }
+#endif
+}
+
 void deleteLastLine() {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
   HIST_ENTRY *entry=remove_history(history_length-1);
