@@ -11,7 +11,7 @@
 #include "config.h"
 #endif
 
-#include "memory.h"
+#include "common.h"
 
 namespace interact {
 
@@ -21,7 +21,14 @@ extern bool uptodate;
 void init_interactive();
   
 // Read a line from the input, without any processing.
-mem::string simpleline();
+string simpleline(string prompt);
+
+// Add a line of input to the readline history.
+void addToHistory(string line);
+
+// Functions to work with the most recently entered line in the history.
+string getLastHistoryLine();
+void setLastHistoryLine(string line);
 
 // Remove the line last added to the history.
 void deleteLastLine();
