@@ -8,7 +8,8 @@
 #ifndef ITEM_H
 #define ITEM_H
 
-#include "memory.h"
+#include "common.h"
+#include <typeinfo>
 
 namespace vm {
 
