@@ -1,10 +1,9 @@
 #ifndef __lexical_h__
 #define __lexical_h__ 1
 
-#include <string>
 #include <sstream>
 
-using std::string;
+#include "common.h"
 
 namespace lexical {
 
@@ -13,7 +12,7 @@ class bad_cast {};
 template <typename T> 
 T cast(const string& s) 
 {
-  std::istringstream is(s);
+  istringstream is(s);
   T value;
   if(is && is >> value && (is >> std::ws).eof()) return value;
   throw bad_cast();
