@@ -1,6 +1,9 @@
 #include <iostream>
 #include <csignal>
 #include <cstdlib>
+#include <cerrno>
+#include <sys/wait.h>
+#include <sys/types.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -20,7 +23,7 @@
 #include "stack.h"
 
 using namespace settings;
-using std::list;
+
 
 errorstream *em;
 using interact::interactive;
@@ -71,6 +74,11 @@ void interruptHandler(int)
   if(em) em->Interrupt(true);
 }
 
+void xasyHandler(int)
+{
+  --signalCount;
+}
+
 //bool status=true;
 
 // Run the config file.
@@ -104,6 +112,9 @@ int main(int argc, char *argv[])
   
   fpu_trap(trap());
 
+  if(getSetting<bool>("signal") && getSetting<double>("deconstruct") != 0.0)
+    signal(SIGUSR1,xasyHandler);
+    
   if(interactive) {
     signal(SIGINT,interruptHandler);
     processPrompt();
@@ -125,5 +136,10 @@ int main(int argc, char *argv[])
   GC_gcollect();
 #endif
 
+  if(getSetting<bool>("wait")) {
+    int status;
+    while(wait(&status) > 0);
+  }
+  
   return em->processStatus() ? 0 : 1;
 }
