@@ -3,7 +3,7 @@
  * Andy Hammerlindl2002/07/14
  *
  * Qualified names (such as x, f, builtin.sin, a.b.c.d, etc.) can be used
- * either as varibles or a type names.  This class stores qualified
+ * either as variables or a type names.  This class stores qualified
  * names used in nameExp and nameTy in the abstract syntax, and
  * implements the exp and type functions.
  *****/
@@ -117,7 +117,7 @@ tyEntry *simpleName::tyEntryTrans(coenv &e)
   if (!ent) {
     em->error(getPos());
     *em << "no type of name \'" << *id << "\'";
-    return new tyEntry(primError(), 0, 0);
+    return new tyEntry(primError(), 0, 0, position());
   }
   return ent;
 }
@@ -286,14 +286,14 @@ tyEntry *qualifiedName::tyEntryTrans(coenv &e)
 
   record *r = castToRecord(rt, false);
   if (!r)
-    return new tyEntry(primError(), 0, 0);
+    return new tyEntry(primError(), 0, 0, position());
 
   tyEntry *ent = r->e.lookupTyEntry(id);
   if (!ent) {
     em->error(getPos());
     *em << "no matching type of name \'" << *id << "\' in \'"
         << *r << "\'";
-    return new tyEntry(primError(), 0, 0);
+    return new tyEntry(primError(), 0, 0, position());
   }
   ent->reportPerm(READ, getPos(), e.c);
 
