@@ -8,17 +8,17 @@
  * three-dimensional algorithms in path3.cc and three.asy.
  *****/
 
-#include <cfloat>
-
 #include "path.h"
 #include "util.h"
 #include "angle.h"
 #include "camperror.h"
 
-static double Fuzz=10.0*DBL_EPSILON;
-  
 namespace camp {
 
+const double Fuzz=10.0*DBL_EPSILON;
+const double Fuzz2=Fuzz*Fuzz;
+const double sqrtFuzz=sqrt(Fuzz);
+
 // Accurate computation of sqrt(1+x)-1.
 inline double sqrt1pxm1(double x)
 {
@@ -261,7 +261,6 @@ pair path::postcontrol(double t) const
   return (bcd == d) ? nodes[iplus].post : bcd;
 }
 
-
 path path::reverse() const
 {
   solvedKnot *nodes = new solvedKnot[n];
@@ -275,28 +274,28 @@ path path::reverse() const
   return path(nodes, n, cycles);
 }
 
-path path::subpath(int start, int end) const
+path path::subpath(int a, int b) const
 {
   if(empty()) return path();
 
-  if (start > end) {
+  if (a > b) {
     const path &rp = reverse();
     int len=length();
-    path result = rp.subpath(len-start, len-end);
+    path result = rp.subpath(len-a, len-b);
     return result;
   }
 
   if (!cycles) {
-    if (start < 0)
-      start = 0;
-    if (end > n-1)
-      end = n-1;
+    if (a < 0)
+      a = 0;
+    if (b > n-1)
+      b = n-1;
   }
 
-  int sn = end-start+1;
+  int sn = b-a+1;
   solvedKnot *nodes = new solvedKnot[sn];
 
-  for (int i = 0, j = start; j <= end; i++, j++) {
+  for (int i = 0, j = a; j <= b; i++, j++) {
     nodes[i].pre = precontrol(j);
     nodes[i].point = point(j);
     nodes[i].post = postcontrol(j);
@@ -322,56 +321,56 @@ inline void splitCubic(solvedKnot sn[], double t, solvedKnot left_,
   mid.point=split(t,mid.pre,mid.post);
 }
 
-path path::subpath(double start, double end) const
+path path::subpath(double a, double b) const
 {
   if(empty()) return path();
   
-  if (start > end) {
+  if (a > b) {
     const path &rp = reverse();
     int len=length();
-    return rp.subpath(len-start, len-end);
+    return rp.subpath(len-a, len-b);
   }
 
-  solvedKnot startL, startR, endL, endR;
+  solvedKnot aL, aR, bL, bR;
   if (!cycles) {
-    if (start < 0) {
-      start = 0;
-      if (end < 0)
-	end = 0;
+    if (a < 0) {
+      a = 0;
+      if (b < 0)
+	b = 0;
     }	
-    if (end > n-1) {
-      end = n-1;
-      if (start > n-1)
-	start = n-1;
+    if (b > n-1) {
+      b = n-1;
+      if (a > n-1)
+	a = n-1;
     }
-    startL = nodes[(int)floor(start)];
-    startR = nodes[(int)ceil(start)];
-    endL = nodes[(int)floor(end)];
-    endR = nodes[(int)ceil(end)];
+    aL = nodes[(int)floor(a)];
+    aR = nodes[(int)ceil(a)];
+    bL = nodes[(int)floor(b)];
+    bR = nodes[(int)ceil(b)];
   } else {
-    if(fabs(start) > INT_MAX || fabs(end) > INT_MAX)
+    if(fabs(a) > INT_MAX || fabs(b) > INT_MAX)
       reportError("invalid path index");
-    startL = nodes[imod((int) floor(start),n)];
-    startR = nodes[imod((int) ceil(start),n)];
-    endL = nodes[imod((int) floor(end),n)];
-    endR = nodes[imod((int) ceil(end),n)];
+    aL = nodes[imod((int) floor(a),n)];
+    aR = nodes[imod((int) ceil(a),n)];
+    bL = nodes[imod((int) floor(b),n)];
+    bR = nodes[imod((int) ceil(b),n)];
   }
 
-  if (start == end) return path(point(start));
+  if (a == b) return path(point(a));
 
   solvedKnot sn[3];
-  path p = subpath(Ceil(start), Floor(end));
-  if (start > floor(start)) {
-    if (end < ceil(start)) {
-      splitCubic(sn,start-floor(start),startL,startR);
-      splitCubic(sn,(end-start)/(ceil(end)-start),sn[1],sn[2]);
+  path p = subpath(Ceil(a), Floor(b));
+  if (a > floor(a)) {
+    if (b < ceil(a)) {
+      splitCubic(sn,a-floor(a),aL,aR);
+      splitCubic(sn,(b-a)/(ceil(b)-a),sn[1],sn[2]);
       return path(sn[0],sn[1]);
     }
-    splitCubic(sn,start-floor(start),startL,startR);
+    splitCubic(sn,a-floor(a),aL,aR);
     p=concat(path(sn[1],sn[2]),p);
   }
-  if (ceil(end) > end) {
-    splitCubic(sn,end-floor(end),endL,endR);
+  if (ceil(b) > b) {
+    splitCubic(sn,b-floor(b),bL,bR);
     p=concat(p,path(sn[0],sn[1]));
   }
   return p;
@@ -388,13 +387,13 @@ static inline void derivative(pair& a, pair& b, pair& c,
 
 bbox path::bounds() const
 {
-  if (empty()) {
-    // No bounds
-    return bbox(/* empty */);
-  }
-
   if(!box.empty) return box;
   
+  if (empty()) {
+    // No bounds
+    return bbox();
+  }
+
   int len=length();
   for (int i = 0; i < len; i++) {
     box += point(i);
@@ -421,23 +420,65 @@ bbox path::bounds() const
   return box;
 }
 
-bbox path::bounds(const bbox& padding) const
+bbox path::bounds(double min, double max) const
 {
-  if (empty()) {
-    // No bounds
-    return bbox(/* empty */);
+  bbox box;
+  
+  static pair I(0,1);
+  
+  int len=length();
+  for (int i = 0; i < len; i++) {
+    pair v=I*dir(i);
+    box += point(i)+min*v;
+    box += point(i)+max*v;
+    if(straight(i)) continue;
+    
+    pair a,b,c;
+    derivative(a,b,c,point(i),postcontrol(i),precontrol(i+1),point(i+1));
+    
+    // Check x coordinate
+    quadraticroots x(a.getx(),b.getx(),c.getx());
+    if(x.distinct != quadraticroots::NONE && goodroot(x.t1)) {
+      double t=i+x.t1;
+      pair v=I*dir(t);
+      box += point(t)+min*v;
+      box += point(t)+max*v;
+    }
+    if(x.distinct == quadraticroots::TWO && goodroot(x.t2)) {
+      double t=i+x.t2;
+      pair v=I*dir(t);
+      box += point(t)+min*v;
+      box += point(t)+max*v;
+    }
+    
+    // Check y coordinate
+    quadraticroots y(a.gety(),b.gety(),c.gety());
+    if(y.distinct != quadraticroots::NONE && goodroot(y.t1)) {
+      double t=i+y.t1;
+      pair v=I*dir(t);
+      box += point(t)+min*v;
+      box += point(t)+max*v;
+    }
+    if(y.distinct == quadraticroots::TWO && goodroot(y.t2)) {
+      double t=i+y.t2;
+      pair v=I*dir(t);
+      box += point(t)+min*v;
+      box += point(t)+max*v;
+    }
   }
-
-  if(!box.empty) return box;
+  pair v=I*dir(len);
+  box += point(len)+min*v;
+  box += point(len)+max*v;
+  return box;
+}
   
-  if(cyclic()) return pad(bounds(),padding);
-  
-  box += point(0);
+bbox path::internalbounds(const bbox& padding) const
+{
+  bbox box;
   
   // Check interior nodes.
   int len=length();
   for (int i = 1; i < len; i++) {
-    box += point(i);
     
     pair pre=point(i)-precontrol(i);
     pair post=postcontrol(i)-point(i);
@@ -489,10 +530,7 @@ bbox path::bounds(const bbox& padding) const
       box += z+pair(0,padding.bottom);
       box += z+pair(0,padding.top);
     }
-    
   }
-  
-  box += point(len);
   return box;
 }
 
@@ -516,12 +554,12 @@ double cubiclength(pair z0, pair z0p, pair z1m, pair z1, double goal=-1)
   if(!simpson(integral,ds,0.0,1.0,DBL_EPSILON,1.0))
     reportError("nesting capacity exceeded in computing arclength");
   L=3.0*integral;
-  if(goal < 0 || goal > L) return L;
+  if(goal < 0 || goal >= L) return L;
   
   static const double third=1.0/3.0;
   goal *= third;
   double t=0.5;
-  if(!unsimpson(goal,ds,0.0,t,100.0*DBL_EPSILON,integral,1.0))
+  if(!unsimpson(goal,ds,0.0,t,10.0*DBL_EPSILON,integral,1.0))
     reportError("nesting capacity exceeded in computing arctime");
   return -t;
 }
@@ -658,12 +696,10 @@ static unsigned count;
 unsigned maxIntersectCount=100000;
 
 // Algorithm derived from Knuth's MetaFont
-pair intersectcubics(solvedKnot left1, solvedKnot right1,
+bool intersectcubics(pair &t, solvedKnot left1, solvedKnot right1,
                      solvedKnot left2, solvedKnot right2,
 		     double fuzz, unsigned depth=DBL_MANT_DIG)
 {
-  const pair F(-1,-1);
-
   bbox box1, box2;
   box1 += left1.point; box1 += left1.post;
   box1 += right1.pre;  box1 += right1.point;
@@ -672,36 +708,45 @@ pair intersectcubics(solvedKnot left1, solvedKnot right1,
   
   double lambda=box1.diameter()+box2.diameter();
   
-  if (box1.Max().getx()+fuzz >= box2.Min().getx() &&
-      box1.Max().gety()+fuzz >= box2.Min().gety() &&
-      box2.Max().getx()+fuzz >= box1.Min().getx() &&
-      box2.Max().gety()+fuzz >= box1.Min().gety()) {
-    if(lambda <= fuzz || depth == 0 || count == 0)
-      return pair(0,0);
+  if(box1.Max().getx()+fuzz >= box2.Min().getx() &&
+     box1.Max().gety()+fuzz >= box2.Min().gety() &&
+     box2.Max().getx()+fuzz >= box1.Min().getx() &&
+     box2.Max().gety()+fuzz >= box1.Min().gety()) {
+    if(lambda <= fuzz || depth == 0 || count == 0) {
+      t=pair(0,0);
+      return true;
+    }
     --depth;
     --count;
     solvedKnot sn1[3], sn2[3];
     splitCubic(sn1,0.5,left1,right1);
     splitCubic(sn2,0.5,left2,right2);
-    pair t;
-    if ((t=intersectcubics(sn1[0],sn1[1],sn2[0],sn2[1],fuzz,depth)) != F)
-      return t*0.5;
-    if ((t=intersectcubics(sn1[0],sn1[1],sn2[1],sn2[2],fuzz,depth)) != F)
-      return t*0.5+pair(0,1);
-    if ((t=intersectcubics(sn1[1],sn1[2],sn2[0],sn2[1],fuzz,depth)) != F)
-      return t*0.5+pair(1,0);
-    if ((t=intersectcubics(sn1[1],sn1[2],sn2[1],sn2[2],fuzz,depth)) != F)
-      return t*0.5+pair(1,1);
+    pair T;
+    if(intersectcubics(T,sn1[0],sn1[1],sn2[0],sn2[1],fuzz,depth)) {
+      t=T*0.5;
+      return true;
+    }
+    if(intersectcubics(T,sn1[0],sn1[1],sn2[1],sn2[2],fuzz,depth)) {
+      t=T*0.5+pair(0,1);
+      return true;
+    }
+    if(intersectcubics(T,sn1[1],sn1[2],sn2[0],sn2[1],fuzz,depth)) {
+      t=T*0.5+pair(1,0);
+      return true;
+    }
+    if(intersectcubics(T,sn1[1],sn1[2],sn2[1],sn2[2],fuzz,depth)) {
+      t=T*0.5+pair(1,1);
+      return true;
+    }
   }
-  return F;
+  return false;
 }
 
 // TODO: Handle corner cases. (Done I think)
-pair intersectiontime(path p1, path p2, double fuzz=0.0)
+bool intersect(pair &t, path p1, path p2, double fuzz=0.0)
 {
   fuzz=max(fuzz,Fuzz*max(max(length(p1.max()),length(p1.min())),
 			 max(length(p2.max()),length(p2.min()))));
-  const pair F(-1,-1);
   solvedKnot *n1=p1.Nodes();
   solvedKnot *n2=p2.Nodes();
   int L1=p1.length();
@@ -710,17 +755,20 @@ pair intersectiontime(path p1, path p2, double fuzz=0.0)
   int jcycle=p2.cyclic() ? p2.size()-1 : -1;
   if(p1.size() == 1) {L1=1; icycle=0;}
   if(p2.size() == 1) {L2=1; jcycle=0;}
-  for (int i = 0; i < L1; i++) {
+  for(int i=0; i < L1; ++i) {
     solvedKnot& left1=n1[i];
     solvedKnot& right1=(i == icycle) ? n1[0] : n1[i+1];
-    for (int j = 0; j < L2; j++) {
+    for(int j=0; j < L2; ++j) {
       count=maxIntersectCount;
-      pair t=intersectcubics(left1,right1,
-			     n2[j],(j == jcycle) ? n2[0] : n2[j+1],fuzz);
-      if (t != F) return t*0.5 + pair(i,j);
+      pair T;
+      if(intersectcubics(T,left1,right1,
+			 n2[j],(j == jcycle) ? n2[0] : n2[j+1],fuzz)) {
+	t=T*0.5+pair(i,j);
+	return true;
+      }
     }
   }
-  return F;  
+  return false;  
 }
 // }}}
 
@@ -768,9 +816,11 @@ path concat(path p1, path p2)
 {
   int n1 = p1.length(), n2 = p2.length();
 
-  if (n1 == 0) return p2;
-  if (n2 == 0) return p1;
-  if (p1.point(n1) != p2.point(0))
+  if (n1 == -1) return p2;
+  if (n2 == -1) return p1;
+  pair a=p1.point(n1);
+  pair b=p2.point(0);
+  if ((a-b).abs2() > Fuzz*max(a.abs2(),b.abs2()))
     reportError("paths in concatenation do not meet");
 
   solvedKnot *nodes = new solvedKnot[n1+n2+1];
@@ -800,10 +850,12 @@ path concat(path p1, path p2)
 bool path::Count(int& count, double t) const
 {
   pair z=point(t);
-  double pre=unit(z-precontrol(t)).gety();
-  double post=unit(postcontrol(t)-z).gety();
-  if(pre == 0.0) pre=post;
-  if(post == 0.0) post=pre;
+  pair Pre=z-precontrol(t);
+  pair Post=postcontrol(t)-z;
+  double pre=unit(Pre).gety();
+  double post=unit(Post).gety();
+  if(pre == 0.0 && Pre != pair(0.0,0.0)) pre=post;
+  if(post == 0.0 && Post != pair(0.0,0.0)) post=pre;
   int incr=(pre*post > Fuzz) ? sgn1(pre) : 0;
   count += incr;
   return incr != 0.0;
@@ -813,15 +865,16 @@ bool path::Count(int& count, double t) const
 void path::countleft(int& count, double x, int i, double t, double begin,
 		     double end, double& mint, double& maxt) const 
 {
+  if(t > -Fuzz && t < Fuzz) t=0;
   if(begin < t && t <= end && x < point(i+t).getx() && Count(count,i+t)) {
     if(t > maxt) maxt=t;
     if(t < mint) mint=t;
   }
 }
 
-// Return the insideness count for the point z relative to the region
-// bounded by the (cyclic) path.
-int path::inside(const pair& z) const
+// Return the winding number of the region bounded by the (cyclic) path
+// relative to the point z.
+int path::windingnumber(const pair& z) const
 {
   if(!cycles)
     reportError("path is not cyclic");
@@ -833,46 +886,46 @@ int path::inside(const pair& z) const
   double begin=-Fuzz;
   double end=1.0+Fuzz;
       
-  double bottom=bounds().bottom;
-  double top=bounds().top;
+  bbox b=bounds();
+  
+  if(z.getx() < b.left || z.getx() > b.right ||
+     z.gety() < b.bottom || z.gety() > b.top) return 0;
   
   for(int i=0; i < n; ++i) {
-    if(z.gety() >= bottom && z.gety() <= top) {
-      pair a=point(i);
-      pair d=point(i+1);
+    pair a=point(i);
+    pair d=point(i+1);
       
-      double mint=1.0;
-      double maxt=0.0;
-      double stop=(i < n-1) ? 1.0+Fuzz : end;
+    double mint=1.0;
+    double maxt=0.0;
+    double stop=(i < n-1) ? 1.0+Fuzz : end;
       
-      if(straight(i)) {
-	double denom=d.gety()-a.gety();
-	if(denom != 0.0)
-	  countleft(count,x,i,(z.gety()-a.gety())/denom,begin,stop,mint,maxt);
-      } else {
-	pair b=postcontrol(i);
-	pair c=precontrol(i+1);
+    if(straight(i)) {
+      double denom=d.gety()-a.gety();
+      if(denom != 0.0)
+	countleft(count,x,i,(z.gety()-a.gety())/denom,begin,stop,mint,maxt);
+    } else {
+      pair b=postcontrol(i);
+      pair c=precontrol(i+1);
     
-	double A=-a.gety()+3.0*(b.gety()-c.gety())+d.gety();
-	double B=3.0*(a.gety()-2.0*b.gety()+c.gety());
-	double C=3.0*(-a.gety()+b.gety());
-	double D=a.gety()-y;
+      double A=-a.gety()+3.0*(b.gety()-c.gety())+d.gety();
+      double B=3.0*(a.gety()-2.0*b.gety()+c.gety());
+      double C=3.0*(-a.gety()+b.gety());
+      double D=a.gety()-y;
     
-	cubicroots r(A,B,C,D);
+      cubicroots r(A,B,C,D);
 
-	if(r.roots >= 1) countleft(count,x,i,r.t1,begin,stop,mint,maxt);
-	if(r.roots >= 2) countleft(count,x,i,r.t2,begin,stop,mint,maxt);
-	if(r.roots >= 3) countleft(count,x,i,r.t3,begin,stop,mint,maxt);
-      }
-      
-      // Avoid double-counting endpoint roots.      
-      if(i == 0)
-	end=camp::min(mint-Fuzz,Fuzz)+1.0;
-      if(mint <= maxt)
-	begin=camp::max(maxt+Fuzz-1.0,-Fuzz); 
-      else // no root found
-	begin=-Fuzz;
+      if(r.roots >= 1) countleft(count,x,i,r.t1,begin,stop,mint,maxt);
+      if(r.roots >= 2) countleft(count,x,i,r.t2,begin,stop,mint,maxt);
+      if(r.roots >= 3) countleft(count,x,i,r.t3,begin,stop,mint,maxt);
     }
+      
+    // Avoid double-counting endpoint roots.      
+    if(i == 0)
+      end=camp::min(mint-Fuzz,Fuzz)+1.0;
+    if(mint <= maxt)
+      begin=camp::max(maxt+Fuzz-1.0,-Fuzz); 
+    else // no root found
+      begin=-Fuzz;
   }
   return count;
 }
