@@ -13,6 +13,7 @@
 
 #include <iostream>
 #include <climits>
+#include <cfloat>
 
 #include "mod.h"
 #include "pair.h"
@@ -22,7 +23,7 @@
 inline double intcap(double t) {
   if(t <= -INT_MAX) return -INT_MAX;
   if(t >= INT_MAX) return INT_MAX;
-    return t;
+  return t;
 }
   
 // The are like floor and ceil, except they return an integer;
@@ -40,9 +41,6 @@ bool unsimpson(double integral, double (*)(double), double a, double& b,
 
 namespace camp {
 
-using std::ostream;
-using std::endl;
-
 // Used in the storage of solved path knots.
 struct solvedKnot : public gc {
   pair pre;
@@ -52,9 +50,12 @@ struct solvedKnot : public gc {
   solvedKnot() : straight(false) {}
 };
 
-
+extern const double Fuzz;
+extern const double Fuzz2;
+extern const double sqrtFuzz;
+  
 class path : public gc {
-  bool cycles;  // If the knot is closed in a loop
+  bool cycles;  // If the path is closed in a loop
 
   int n; // The number of knots
 
@@ -133,68 +134,118 @@ public:
       reportError("nullpath has no points");
   }
   
-  pair point(int i) const
+  pair point(int t) const
   {
     emptyError();
     
     if (cycles)
-      return nodes[imod(i,n)].point;
-    else if (i < 0)
+      return nodes[imod(t,n)].point;
+    else if (t < 0)
       return nodes[0].point;
-    else if (i >= n)
+    else if (t >= n)
       return nodes[n-1].point;
     else
-      return nodes[i].point;
+      return nodes[t].point;
   }
 
-  bool straight(int i) const
+  bool straight(int t) const
   {
-    if (cycles) return nodes[imod(i,n)].straight;
-    return (i >= 0 && i < n) ? nodes[i].straight : false;
+    if (cycles) return nodes[imod(t,n)].straight;
+    return (t >= 0 && t < n) ? nodes[t].straight : false;
   }
   
   pair point(double t) const;
   
-  pair precontrol(int i) const
+  pair precontrol(int t) const
   {
     emptyError();
 		       
     if (cycles)
-      return nodes[imod(i,n)].pre;
-    else if (i < 0)
+      return nodes[imod(t,n)].pre;
+    else if (t < 0)
       return nodes[0].pre;
-    else if (i >= n)
+    else if (t >= n)
       return nodes[n-1].pre;
     else
-      return nodes[i].pre;
+      return nodes[t].pre;
   }
 
   pair precontrol(double t) const;
   
-  pair postcontrol(int i) const
+  pair postcontrol(int t) const
   {
     emptyError();
 		       
     if (cycles)
-      return nodes[imod(i,n)].post;
-    else if (i < 0)
+      return nodes[imod(t,n)].post;
+    else if (t < 0)
       return nodes[0].post;
-    else if (i >= n)
+    else if (t >= n)
       return nodes[n-1].post;
     else
-      return nodes[i].post;
+      return nodes[t].post;
   }
 
   pair postcontrol(double t) const;
   
-  pair direction(int i) const
-  {
-    return postcontrol(i) - precontrol(i);
+  pair predir(int t) const {
+    if(!cycles && t <= 0) return pair(0,0);
+    pair z0=point(t-1);
+    pair z1=point(t);
+    pair c1=precontrol(t);
+    pair dir=z1-c1;
+    double epsilon=Fuzz2*(z0-z1).abs2();
+    if(dir.abs2() > epsilon) return unit(dir);
+    pair c0=postcontrol(t-1);
+    dir=2*c1-c0-z1;
+    if(dir.abs2() > epsilon) return unit(dir);
+    return unit(z1-z0+3*(c0-c1));
   }
 
-  pair direction(double t) const
-  {
-    return postcontrol(t) - precontrol(t);
+  pair predir(double t) const {
+    if(!cycles) {
+      if(t <= 0) return pair(0,0);
+      if(t >= n-1) return predir(n-1);
+    }
+    int a=Floor(t);
+    return (t-a < sqrtFuzz) ? predir(a) : subpath((double) a,t).predir(1);
+  }
+
+  pair postdir(int t) const {
+    if(!cycles && t >= n-1) return pair(0,0);
+    pair z0=point(t);
+    pair z1=point(t+1);
+    pair c0=postcontrol(t);
+    pair dir=c0-z0;
+    double epsilon=Fuzz2*(z0-z1).abs2();
+    if(dir.abs2() > epsilon) return unit(dir);
+    pair c1=precontrol(t+1);
+    dir=z0-2*c0+c1;
+    if(dir.abs2() > epsilon) return unit(dir);
+    return unit(z1-z0+3*(c0-c1));
+  }
+
+  pair postdir(double t) const {
+    if(!cycles) {
+      if(t >= n-1) return pair(0,0);
+      if(t <= 0) return postdir(0);
+    }
+    int b=Ceil(t);
+    return (b-t < sqrtFuzz) ? postdir(b) : subpath(t,(double) b).postdir(0);
+  }
+
+  pair dir(int t) const {
+    return unit(predir(t)+postdir(t));
+  }
+  
+  pair dir(double t) const {
+    return unit(predir(t)+postdir(t));
+  }
+
+  pair dir(int t, int sign) const {
+    if(sign == 0) return dir(t);
+    else if(sign > 0) return postdir(t);
+    else return predir(t);
   }
 
   // Returns the path traced out in reverse.
@@ -206,27 +257,28 @@ public:
   path subpath(double start, double end) const;
 
   // Used by picture to determine bounding box
-  // NOTE: Conservative here; uses only control points.
-  //       A better method is in the MetaPost source.
   bbox bounds() const;
   
-  // Return bounding box accounting for internal pen padding (but not pencap).
-  bbox bounds(const bbox &padding) const;
+  // Return bounding box accounting for padding perpendicular to path.
+  bbox bounds(double min, double max) const;
 
+  // Return bounding box accounting for internal pen padding (but not pencap).
+  bbox internalbounds(const bbox &padding) const;
+  
   double arclength () const;
   double arctime (double l) const;
   double directiontime(pair z) const;
  
-  pair max () {
+  pair max() {
     return bounds().Max();
   }
 
-  pair min () {
+  pair min() {
     return bounds().Min();
   }
   
   // Debugging output
-  friend ostream& operator<< (ostream& out, const path p);
+  friend std::ostream& operator<< (std::ostream& out, const path p);
 
   int sgn1(double x) const
   {
@@ -240,16 +292,16 @@ public:
   void countleft(int& count, double x, int i, double t,
 		 double begin, double end, double& mint, double& maxt) const;
 
-// Return the insideness count for the point z relative to the region
-// bounded by the (cyclic) path.
-  int inside(const pair& z) const;
+// Return the winding number of the region bounded by the (cyclic) path
+// relative to the point z.
+  int windingnumber(const pair& z) const;
 
   // Transformation
   path transformed(const transform& t) const;
   
 };
 
-pair intersectiontime(path p1, path p2, double fuzz);
+bool intersect(pair& t, path p1, path p2, double fuzz);
   
 // Concatenates two paths into a new one.
 path concat(path p1, path p2);
