@@ -21,17 +21,23 @@ using namespace settings;
 
 namespace camp {
 
-const char *texpathmessage="the directory containing your tex engine (latex by default)";
+const char *texpathmessage() {
+  ostringstream buf;
+  buf << "the directory containing your " << getSetting<string>("tex")
+      << " engine (" << texengine() << ")";
+  return strcpy(new char[buf.str().size()+1],buf.str().c_str());
+}
   
-string texready=string("(Please type a command or say `\\end')\n*");
 texstream tex; // Bi-directional pipe to latex (to find label bbox)
 
 void texstream::pipeclose() {
   iopipestream::pipeclose();
-  if (!getSetting<bool>("keep")) {
+  if(!getSetting<bool>("keep")) {
     unlink("texput.log");
     unlink("texput.out");
     unlink("texput.aux");
+    if(settings::pdf(texengine()))
+      unlink("texput.pdf");
   }
 }
 
@@ -39,7 +45,7 @@ picture::~picture()
 {
 }
 
-bool picture::epsformat,picture::pdfformat,picture::tgifformat, picture::pdf;
+bool picture::epsformat,picture::pdfformat,picture::xobject, picture::pdf;
 bool picture::Labels;
 double picture::paperWidth,picture::paperHeight;
   
@@ -96,7 +102,7 @@ void picture::prepend(picture &pic)
 bool picture::havelabels()
 {
   size_t n=nodes.size();
-  if(n > lastnumber && !labels && getSetting<mem::string>("tex") != "none") {
+  if(n > lastnumber && !labels && getSetting<string>("tex") != "none") {
     // Check to see if there are any labels yet
     nodelist::iterator p=nodes.begin();
     for(size_t i=0; i < lastnumber; ++i) ++p;
@@ -137,23 +143,22 @@ void picture::texinit()
 {
   drawElement::lastpen=pen(initialpen);
   // Output any new texpreamble commands
-  if(TeXinitialized) {
+  if(tex.isopen()) {
     if(TeXpipepreamble.empty()) return;
     texpreamble(tex,TeXpipepreamble);
     TeXpipepreamble.clear();
     return;
   }
   
-
-  tex.open(texengine().c_str(),"texpath",texpathmessage);
+  ostringstream cmd;
+  cmd << "'" << texprogram() << "'" << " \\scrollmode";
+  tex.open(cmd.str().c_str(),"texpath",texpathmessage());
+  tex.wait("\n*");
+  tex << "\n";
   texdocumentclass(tex,true);
   
-  texdefines(tex,TeXpipepreamble,true);
+  texdefines(tex,TeXpreamble,true);
   TeXpipepreamble.clear();
-
-  tex << "\n";
-  tex.wait(texready.c_str(),"! ");
-  TeXinitialized=true;
 }
   
 bool picture::texprocess(const string& texname, const string& outname,
@@ -166,23 +171,18 @@ bool picture::texprocess(const string& texname, const string& outname,
   if(outfile) {
     outfile.close();
     ostringstream cmd;
-    cmd << "'" << texengine() << "'"
-	<< " \\scrollmode\\input '" << texname << "'";
+    cmd << "'" << texprogram() << "'"
+	<< " \\nonstopmode\\input '" << texname << "'";
     bool quiet=verbose <= 1;
-    status=System(cmd,quiet ? 1 : 0,"texpath",texpathmessage);
-    if(!status && getSetting<bool>("twice")) 
-      status=System(cmd,quiet ? 1 : 0,"texpath",texpathmessage);
+    status=System(cmd,quiet ? 1 : 0,"texpath",texpathmessage());
+    if(!status && getSetting<bool>("twice"))
+      status=System(cmd,quiet ? 1 : 0,"texpath",texpathmessage());
     if(status) {
       if(quiet) System(cmd,0);
       return false;
     }
     
-    mem::string texengine=getSetting<mem::string>("tex");
-    if(settings::pdf(texengine)) {
-      if(pdfformat) {
-	rename(outname.c_str(),buildname(prefix,"pdf").c_str());
-      }
-    } else {
+    if(!pdf) {
       string dviname=auxname(prefix,"dvi");
       string psname=auxname(prefix,"ps");
     
@@ -191,14 +191,15 @@ bool picture::texprocess(const string& texname, const string& outname,
       // Magic dvips offsets:
       double hoffset=-128.4;
       double vertical=height;
-      if(!settings::latex(texengine)) vertical += 2.0;
+      if(!latex(getSetting<string>("tex"))) vertical += 2.0;
       double voffset=(vertical < 13.0) ? -137.8+vertical : -124.8;
 
       hoffset += b.left+bboxshift.getx();
       voffset += paperHeight-height-b.bottom-bboxshift.gety();
     
       ostringstream dcmd;
-      dcmd << "'" << getSetting<mem::string>("dvips") << "' -R "
+      dcmd << "'" << getSetting<string>("dvips") << "' "
+	   << getSetting<string>("dvipsOptions") << " -R "
 	   << " -O " << hoffset << "bp," << voffset << "bp"
 	   << " -T " << paperWidth << "bp," << paperHeight << "bp";
       if(verbose <= 1) dcmd << " -q";
@@ -217,7 +218,9 @@ bool picture::texprocess(const string& texname, const string& outname,
       string endspecial="@fedspecial end";
       while(getline(fin,s)) {
 	if(s.find("%%DocumentPaperSizes:") == 0) continue;
-	if(first && s.find("%%BoundingBox:") == 0) {
+	if(s.find("%!PS-Adobe-") == 0) {
+	  fout.verbatimline("%!PS-Adobe-3.0 EPSF-3.0");
+	} else if(first && s.find("%%BoundingBox:") == 0) {
 	  bbox box=b;
 	  box.shift(bboxshift);
 	  if(verbose > 2) BoundingBox(cout,box);
@@ -241,7 +244,7 @@ bool picture::texprocess(const string& texname, const string& outname,
       
     if(!getSetting<bool>("keep")) { // Delete temporary files.
       unlink(texname.c_str());
-      unlink(auxname(prefix,"aux").c_str());
+      if(!getSetting<bool>("keepaux")) unlink(auxname(prefix,"aux").c_str());
       unlink(auxname(prefix,"log").c_str());
       unlink(auxname(prefix,"out").c_str());
     }
@@ -254,19 +257,19 @@ int picture::epstopdf(const string& epsname, const string& pdfname)
 {
   ostringstream cmd;
   
-  cmd << "'" << getSetting<mem::string>("gs")
+  cmd << "'" << getSetting<string>("gs")
       << "' -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite -dEPSCrop"
       << " -dCompatibilityLevel=1.4";
   if(!getSetting<bool>("autorotate"))
     cmd << " -dAutoRotatePages=/None";
-  cmd << " -g" << ceil(paperWidth) << "x" << ceil(paperHeight)
-      << " -dDEVICEWIDTHPOINTS=" << b.right-b.left
-      << " -dDEVICEHEIGHTPOINTS=" << b.top-b.bottom
+  cmd << " -g" << max(ceil(paperWidth),1.0) << "x" << max(ceil(paperHeight),1.0)
+      << " -dDEVICEWIDTHPOINTS=" << max(b.right-b.left,3.0)
+      << " -dDEVICEHEIGHTPOINTS=" << max(b.top-b.bottom,3.0)
       << " -sOutputFile='" << pdfname << "' '" << epsname << "'";
   return System(cmd,0,true,"gs","Ghostscript");
 }
   
-std::map<CONST std::string,int> pids;
+std::map<CONST string,int> pids;
 
 bool picture::postprocess(const string& prename, const string& outname,
 			  const string& outputformat, bool wait, bool view)
@@ -275,17 +278,26 @@ bool picture::postprocess(const string& prename, const string& outname,
   
   if((pdf && Labels) || !epsformat) {
     if(pdfformat) {
-      if(!(pdf && Labels)) status=epstopdf(prename,outname);
+      if(pdf && Labels) status=rename(prename.c_str(),outname.c_str());
+      else status=epstopdf(prename,outname);
+    } else if(xobject && getSetting<string>("xformat") == "png") {
+      ostringstream cmd;
+      double res=getSetting<double>("deconstruct")*72.0;
+      cmd << "'" << getSetting<string>("gs")
+	  << "' -q -dNOPAUSE -dBATCH -sDEVICE=pngalpha -dEPSCrop"
+	  << " -r" << res << "x" << res
+	  << " -sOutputFile='" << outname << "' '" << prename << "'";
+      status=System(cmd,0,true,"gs","Ghostscript");
     } else {
       ostringstream cmd;
       double expand=2.0;
-      double res=(tgifformat ? getSetting<double>("deconstruct") : expand)*
+      double res=(xobject ? getSetting<double>("deconstruct") : expand)*
 	72.0;
-      cmd << "'" << getSetting<mem::string>("convert") 
+      cmd << "'" << getSetting<string>("convert") 
 	  << "' -density " << res << "x" << res;
-      if(!tgifformat) cmd << " +antialias -geometry " << 100.0/expand << "%x";
+      if(!xobject) cmd << " +antialias -geometry " << 100.0/expand << "%x";
       cmd << " '" << (pdf ? "pdf:" : "eps:") << prename << "'";
-      if(tgifformat) cmd << " -transparent white gif";
+      if(xobject) cmd << " -transparent white gif";
       else cmd << " " << outputformat;
       cmd << ":'" << outname << "'";
       status=System(cmd,0,true,"convert");
@@ -294,15 +306,15 @@ bool picture::postprocess(const string& prename, const string& outname,
   }
   if(status != 0) return false;
   
-  if(verbose > (tgifformat ? 1 : 0)) 
+  if(verbose > (xobject ? 1 : 0)) 
     cout << "Wrote " << outname << endl;
   if(settings::view() && view) {
     if(epsformat || pdfformat) {
       // Check to see if there is an existing viewer for this outname.
-      std::map<CONST std::string,int>::iterator p=pids.find(outname);
+      std::map<CONST string,int>::iterator p=pids.find(outname);
       bool running=(p != pids.end());
-      string Viewer=pdfformat ? getSetting<mem::string>("pdfviewer") :
-	getSetting<mem::string>("psviewer");
+      string Viewer=pdfformat ? getSetting<string>("pdfviewer") :
+	getSetting<string>("psviewer");
       int pid;
       if(running) {
 	pid=p->second;
@@ -313,8 +325,6 @@ bool picture::postprocess(const string& prename, const string& outname,
       if(!running) {
 	ostringstream cmd;
 	cmd << "'" << Viewer << "'";
-	if(Viewer == "gv" && interact::interactive)
-	  cmd << " " << gvOptionPrefix << "nowatch";
 	cmd << " '" << outname << "'";
 	status=System(cmd,0,wait,
 		      pdfformat ? "pdfviewer" : "psviewer",
@@ -325,7 +335,7 @@ bool picture::postprocess(const string& prename, const string& outname,
       } else if(Viewer == "gv") kill(pid,SIGHUP); // Tell gv to reread file.
     } else {
       ostringstream cmd;
-      cmd << "'" << getSetting<mem::string>("display") << "' '"
+      cmd << "'" << getSetting<string>("display") << "' '"
 	  << outname << "'";
       string application="your "+outputformat+" viewer";
       status=System(cmd,0,wait,"display",application.c_str());
@@ -342,23 +352,21 @@ bool picture::shipout(picture *preamble, const string& Prefix,
   bounds();
   
   bool TeXmode=getSetting<bool>("inlinetex") && 
-    getSetting<mem::string>("tex") != "none";
-  Labels=(labels || TeXmode) && (b.right > b.left && b.top > b.bottom);
+    getSetting<string>("tex") != "none";
+  Labels=labels || TeXmode;
   
-  pdf=settings::pdf(getSetting<mem::string>("tex"));
+  pdf=settings::pdf(getSetting<string>("tex"));
   
   bool standardout=Prefix == "-";
   string prefix=standardout ? "out" : Prefix;
-  checkFormatString(format);
   string preformat=nativeformat();
-  string defaultformat=(string) getSetting<mem::string>("outformat");
-  if(defaultformat == "") defaultformat=preformat;
-  string outputformat=format.empty() ? defaultformat : format;
+  string outputformat=format.empty() ? defaultformat() : format;
   epsformat=outputformat == "eps";
   pdfformat=outputformat == "pdf";
-  tgifformat=outputformat == "tgif";
-  string outname=tgifformat ? "."+buildname(prefix,"gif") :
-    (standardout ? "-" : buildname(prefix,outputformat,"",false));
+  xobject=outputformat == "<x>";
+  string xformat=getSetting<string>("xformat");
+  string outname=xobject ? "."+buildname(prefix,xformat) :
+    (standardout ? "-" : buildname(prefix,outputformat,"",!global()));
   string epsname=epsformat ? (standardout ? "" : outname) :
     auxname(prefix,"eps");
   string prename=((epsformat && !pdf) || !Labels) ? epsname : 
@@ -367,14 +375,15 @@ bool picture::shipout(picture *preamble, const string& Prefix,
   
   static ofstream bboxout;
   
-  if(null()) { // Output a null file
+  if(b.empty && !Labels) { // Output a null file
     bbox b;
     b.left=b.bottom=0;
     b.right=b.top=1;
     psfile out(epsname,false);
     out.prologue(b);
     out.epilogue();
-    if(deconstruct && !tgifformat) {
+    out.close();
+    if(deconstruct && !xobject) {
       if(bboxout) bboxout.close();
       ShipoutNumber++;
       return true;
@@ -382,36 +391,45 @@ bool picture::shipout(picture *preamble, const string& Prefix,
     return postprocess(epsname,outname,outputformat,wait,view);
   }
   
-  if(deconstruct && !tgifformat) {
-    if(bboxout) bboxout.close();
-    if(settings::view() && view) {
-      ostringstream cmd;
-      string Python=getSetting<mem::string>("python");
-      if(Python != "") cmd << "'" << Python << "' ";
-      cmd << "'" << getSetting<mem::string>("xasy") << "' " 
-	  << buildname(prefix) << " " << ShipoutNumber << " "
-	  << buildname(settings::outname());
-      int status=System(cmd,0,true,Python != "" ? "python" : "xasy");
-      if(status != 0) return false;
-    }
-    ShipoutNumber++;
-    return true;
-  }
-      
   if(deconstruct) {
-      if(!bboxout.is_open()) {
-	bboxout.open(("."+buildname(prefix,"box")).c_str());	
-	bboxout << deconstruct << newl;
-      }
+    bool signal=getSetting<bool>("signal");
+    if(!bboxout.is_open()) {
+      bboxout.open(("."+buildname(prefix,"box")).c_str());	
+      bboxout << (xobject ? deconstruct : 0) << " " << xformat << newl;
+    }
+    if(xobject) {
       bbox bscaled=b;
+      // Work around half-pixel bounding box bug in Ghostscript pngalpha driver
+      if(xformat == "png") {
+	double fuzz=0.5/deconstruct;
+	b.top += fuzz;
+	b.right += fuzz;
+      }
       bscaled *= deconstruct;
-      bboxout << bscaled << newl;
+      bboxout << bscaled << endl;
+      if(signal) bboxout.close();
       if(Delete) {
 	unlink(outname.c_str());
 	return false;
       }
+    } else {
+      if(bboxout) bboxout.close();
+      if(settings::view() && view) {
+	ostringstream cmd;
+	string Python=getSetting<string>("python");
+	if(Python != "") cmd << "'" << Python << "' ";
+	cmd << "'" << getSetting<string>("xasy") << "' " 
+	    << buildname(prefix) << " " << ShipoutNumber << " "
+	    << buildname(settings::outname());
+	int status=System(cmd,0,true,Python != "" ? "python" : "xasy");
+	if(status != 0) return false;
+      }
+      ShipoutNumber++;
+      return true;
+    }
   }
-  
+
+      
   SetPageDimensions();
   
   paperWidth=getSetting<double>("paperwidth");
@@ -447,7 +465,7 @@ bool picture::shipout(picture *preamble, const string& Prefix,
   nodelist::iterator layerp=nodes.begin();
   nodelist::iterator p=layerp;
   unsigned int layer=0;
-  std::list<string> psnameStack;
+  mem::list<string> psnameStack;
   
   bbox bshift=b;
   
@@ -497,14 +515,13 @@ bool picture::shipout(picture *preamble, const string& Prefix,
     if(!Labels) out.grestore();
     
     out.epilogue();
+    out.close();
     
     if(Labels) {
       tex->resetpen();
       if(status) {
-	if(pdf) {
-	  out.close();
+	if(pdf && !b.empty)
 	  status=(epstopdf(psname,pdfname) == 0);
-	}
 	
 	if(status) {
 	  for (p=layerp; p != nodes.end(); ++p) {
@@ -533,7 +550,7 @@ bool picture::shipout(picture *preamble, const string& Prefix,
 	status=texprocess(texname,prename,prefix,bboxshift);
 	delete tex;
 	if(!getSetting<bool>("keep")) {
-	  for(std::list<string>::iterator p=psnameStack.begin();
+	  for(mem::list<string>::iterator p=psnameStack.begin();
 	      p != psnameStack.end(); ++p)
 	    unlink(p->c_str());
 	}
