@@ -9,7 +9,6 @@
 #ifndef PICTURE_H
 #define PICTURE_H
 
-#include <list>
 #include <sstream>
 #include <iostream>
 
@@ -34,7 +33,7 @@ private:
   boxvector labelbounds;
   bboxlist bboxstack;
   
-  static bool epsformat,pdfformat,tgifformat,pdf,Labels;
+  static bool epsformat,pdfformat,xobject,pdf,Labels;
   static double paperWidth,paperHeight;
 
 public:
@@ -82,10 +81,6 @@ public:
     return nodes.empty();
   }
   
-  bool empty() {
-    bounds();
-    return null();
-  }
 };
 
 inline picture *transformed(const transform& t, picture *p)
