@@ -1,39 +1,37 @@
 /* Pipestream: A simple C++ interface to UNIX pipes
    Version 0.01
-   Copyright (C) 2005 John C. Bowman
+   Copyright (C) 2005-2006 John C. Bowman
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
 #ifndef PIPESTREAM_H
 #define PIPESTREAM_H
 
 #include <iostream>
-#include <string>
 #include <sys/wait.h>
 #include <cerrno>
 #include <sstream>
 #include <unistd.h>
 #include <signal.h>
 
+#include "common.h"
 #include "errormsg.h"
 #include "settings.h"
 #include "util.h"
 #include "interact.h"
 
-using std::string;
-
 char **args(const char *command);
   
 // bidirectional stream for reading and writing to pipes
@@ -45,7 +43,19 @@ protected:
   char buffer[BUFSIZE];
   int pid;
   bool pipeopen;
+  string *last;
 public:
+  
+  void clear() {
+    if(last) delete last;
+    last=new string(buffer);
+    *buffer=0;
+  }
+  
+  const string *message() {
+    return last;
+  }
+  
   void open(const char *command, const char *hint=NULL,
 	    const char *application="", int out_fileno=STDOUT_FILENO) {
     if(pipe(in) == -1) {
@@ -93,12 +103,15 @@ public:
     } else {
       close(out[1]);
       close(in[0]);
+      *buffer=0;
       pipeopen=true;
       waitpid(wrapperpid,NULL,0);
     }
   }
 
-  iopipestream(): pid(0), pipeopen(false) {}
+  bool isopen() {return pipeopen;}
+  
+  iopipestream(): pid(0), pipeopen(false), last(NULL) {}
   
   iopipestream(const char *command, const char *hint=NULL,
 	       const char *application="", int out_fileno=STDOUT_FILENO) :
@@ -129,7 +142,7 @@ public:
       p[nc]=0;
       if(nc == 0) break;
       if(nc > 0) {
-	if(settings::verbose > 2) std::cerr << p << std::endl;
+	if(settings::verbose > 2) cerr << p;
 	if(strchr(p,'\n')) break;
 	p += nc;
 	size -= nc;
@@ -160,26 +173,42 @@ public:
     return true;
   }
   
-  void wait(const char *prompt, const char *abort=NULL) {
+  bool checkabort(const char *abort) {
+    size_t alen=strlen(abort);
+    if(strncmp(buffer,abort,alen) == 0) {
+      clear();
+      return true;
+    }
+    char *p=buffer;
+    while((p=strchr(p,'\n')) != NULL) {
+      ++p;
+      if(strncmp(p,abort,alen) == 0) {
+	clear();
+	return true;
+      }
+    }
+    return false;
+  }
+  
+  // returns true if prompt found, false if abort string is received
+  int wait(const char *prompt, const char**abort=NULL) {
     ssize_t len;
     size_t plen=strlen(prompt);
-    size_t alen=0;
-    if(abort) alen=strlen(abort);
+  
+    unsigned int n=0;
+    if(abort) {
+      for(;;) {
+	if(abort[n]) n++;
+	else break;
+      }
+    }
+    
     do {
       len=readbuffer();
-      if(abort) {
-	if(strncmp(buffer,abort,alen) == 0) {
-	  if(settings::getSetting<bool>("inlinetex")) return;
-	  camp::reportError(buffer);
-	}
-	char *p=buffer;
-	while((p=strchr(p,'\n')) != NULL) {
-	  ++p;
-	  if(strncmp(p,abort,alen) == 0)
-	    camp::reportError(buffer);
-	}
-      }
+      for(unsigned int i=0; i < n; ++i) 
+	if(checkabort(abort[i])) return i+1;
     } while (!tailequals(buffer,len,prompt,plen));
+    return 0;
   }
 
   int wait() {
@@ -203,22 +232,25 @@ public:
     }
   }
   
-  iopipestream& operator << (const string &s) {
+  void Write(const string &s) {
     ssize_t size=s.length();
-    if(settings::verbose > 2) std::cerr << s << std::endl;
+    if(settings::verbose > 2) cerr << s;
     if(write(in[1],s.c_str(),size) != size)
       camp::reportError("write to pipe failed");
+  }
+  
+  iopipestream& operator << (const string& s) {
+    Write(s);
     return *this;
   }
   
   template<class T>
   iopipestream& operator << (T x) {
-    std::ostringstream os;
+    ostringstream os;
     os << x;
-    *this << os.str();
+    Write(os.str());
     return *this;
   }
 };
   
 #endif
-
