@@ -25,6 +25,8 @@
 
 #include "process.h"
 
+int signalCount=0;
+
 namespace run {
   void cleanup();
   void exitFunction(vm::stack *Stack);
@@ -36,8 +38,6 @@ namespace vm {
 }
 
 using namespace settings;
-using std::list;
-using std::string;
 
 using absyntax::file;
 using trans::genv;
@@ -49,11 +49,12 @@ using types::record;
 using interact::interactive;
 using interact::uptodate;
 
-void init()
+void init(bool resetpath=true)
 {
   vm::indebugger=false;
-  setPath(startPath());  /* On second and subsequent calls to init, sets the
-                            path to what it was when the program started. */
+  if(resetpath)
+    setPath("");  /* On second and subsequent calls to init, sets the
+		     path to what it was when the program started. */
   ShipoutNumber=0;
   if(!em)
     em = new errorstream();
@@ -69,17 +70,52 @@ void purge()
 using absyntax::runnable;
 using absyntax::block;
 
+// This helper class does nothing but call the interactiveTrans method of the
+// base object in place of trans, so that the runnable can exhibit special
+// behaviour when run at the interactive prompt.
+class interactiveRunnable : public runnable {
+  runnable *base;
+
+public:
+  interactiveRunnable(runnable *base)
+    : runnable(base->getPos()), base(base) {}
+
+  void prettyprint(ostream &out, int indent) {
+    absyntax::prettyname(out, "interactiveRunnable", indent);
+    base->prettyprint(out, indent+1);
+  }
+  
+  void trans(coenv &e) {
+    base->interactiveTrans(e);
+  }
+
+  void transAsField(coenv &e, types::record *r) {
+    // There is no interactiveTransAsField, as fields aren't declared at the top
+    // level of the interactive prompt.
+    base->transAsField(e, r);
+  }
+};
+  
+enum transMode {
+  TRANS_INTERACTIVE,
+  TRANS_NORMAL
+};
+
+
 // How to run a runnable in runnable-at-a-time mode.
-bool runRunnable(runnable *r, coenv &e, istack &s) {
+bool runRunnable(runnable *r, coenv &e, istack &s, transMode tm=TRANS_NORMAL) {
   e.e.beginScope();
-  lambda *codelet=r->transAsCodelet(e);
+
+  lambda *codelet= tm==TRANS_INTERACTIVE ?
+                       interactiveRunnable(r).transAsCodelet(e) :
+                       r->transAsCodelet(e);
   em->sync();
   if(!em->errors()) {
     if(getSetting<bool>("translate")) print(cout,codelet->code);
     s.run(codelet);
 
-    // NOTE: May want to add a "e.e.collapseTopScope()" here to keep scoping
-    // from getting out-of-hand.
+    // Commits the changes made to the environment.
+    e.e.collapseScope();
   } else {
     e.e.endScope(); // Remove any changes to the environment.
 
@@ -105,25 +141,24 @@ struct icore {
   virtual void doParse() = 0;
   virtual void doList() = 0;
 
-private:
+protected:
   // NOTE: Get this out of here!
-  std::list<string> TeXpipepreamble_save;
-  std::list<string> TeXpreamble_save;
+  mem::list<string> TeXpipepreamble_save;
+  mem::list<string> TeXpreamble_save;
 public:
 
   virtual void preRun(coenv &e, istack &s) {
-    TeXpipepreamble_save = std::list<string>(camp::TeXpipepreamble);
-    TeXpreamble_save = std::list<string>(camp::TeXpreamble);
+    TeXpipepreamble_save = mem::list<string>(camp::TeXpipepreamble);
+    TeXpreamble_save = mem::list<string>(camp::TeXpreamble);
 
     if(getSetting<bool>("autoplain"))
       runAutoplain(e,s);
   }
 
-  virtual void run(coenv &e, istack &s) = 0;
+  virtual void run(coenv &e, istack &s, transMode tm=TRANS_NORMAL) = 0;
 
   virtual void postRun(coenv &, istack &s) {
-    if(interactive) run::cleanup();
-    else run::exitFunction(&s);
+    run::exitFunction(&s);
     camp::TeXpipepreamble=TeXpipepreamble_save;
     camp::TeXpreamble=TeXpreamble_save;
   }
@@ -153,10 +188,10 @@ public:
 
       postRun(e,s);
 
-    } catch (std::bad_alloc&) {
+    } catch(std::bad_alloc&) {
       cerr << "error: out of memory" << endl;
       em->statusError();
-    } catch (handled_error) {
+    } catch(handled_error) {
       em->statusError();
       run::cleanup();
     }
@@ -176,11 +211,11 @@ public:
 
 // Abstract base class for one-time processing of an abstract syntax tree.
 class itree : public icore {
-  mem::string name;
+  string name;
 
   block *cachedTree;
 public:
-  itree(mem::string name="<unnamed>")
+  itree(string name="<unnamed>")
     : name(name), cachedTree(0) {}
 
   // Build the tree, possibly throwing a handled_error if it cannot be built.
@@ -190,7 +225,7 @@ public:
     if (cachedTree==0) {
       try {
         cachedTree=buildTree();
-      } catch (handled_error) {
+      } catch(handled_error) {
         em->statusError();
         return 0;
       }
@@ -198,7 +233,7 @@ public:
     return cachedTree;
   }
 
-  virtual mem::string getName() {
+  virtual string getName() {
     return name;
   }
 
@@ -218,13 +253,13 @@ public:
     }
   }
 
-  void run(coenv &e, istack &s) {
+  void run(coenv &e, istack &s, transMode tm=TRANS_NORMAL) {
     block *tree=getTree();
     if (tree) {
-      for(list<runnable *>::iterator r=tree->stms.begin();
+      for(mem::list<runnable *>::iterator r=tree->stms.begin();
           r != tree->stms.end(); ++r)
         if(!em->errors() || getSetting<bool>("debug"))
-          runRunnable(*r,e,s);
+          runRunnable(*r,e,s,tm);
     }
   }
 
@@ -240,7 +275,7 @@ class icode : public itree {
   block *tree;
 
 public:
-  icode(block *tree, mem::string name="<unnamed>")
+  icode(block *tree, string name="<unnamed>")
     : itree(name), tree(tree) {}
 
   block *buildTree() {
@@ -249,10 +284,10 @@ public:
 };
 
 class istring : public itree {
-  mem::string str;
+  string str;
 
 public:
-  istring(const mem::string& str, mem::string name="<eval>")
+  istring(const string& str, string name="<eval>")
     : itree(name), str(str) {}
 
   block *buildTree() {
@@ -261,23 +296,23 @@ public:
 };
 
 class ifile : public itree {
-  mem::string filename;
-  mem::string outname;
-  mem::string outname_save;
+  string filename;
+  string outname;
+  string outname_save;
 
 public:
-  ifile(const mem::string& filename)
+  ifile(const string& filename)
     : itree(filename),
       filename(filename),
-      outname((mem::string) (filename == "-" ? "out" :
-			     stripDir(stripExt(string(filename),suffix)))) {}
+      outname((string) (filename == "-" ? "out" :
+			stripDir(stripExt(string(filename), suffix)))) {}
   
   block *buildTree() {
     return filename!="" ? parser::parseFile(filename) : 0;
   }
 
   void preRun(coenv& e, istack& s) {
-    outname_save=getSetting<mem::string>("outname");
+    outname_save=getSetting<string>("outname");
     if(outname_save.empty())
       Setting("outname")=outname;
 
@@ -291,7 +326,10 @@ public:
   }
 
   void process() {
-    init();
+    try {
+      init();
+    } catch(handled_error) {
+    }
 
     if (verbose >= 1)
       cout << "Processing " << outname << endl;
@@ -299,7 +337,7 @@ public:
     try {
       itree::process();
     }
-    catch (handled_error) {
+    catch(handled_error) {
       em->statusError();
     }
   }
@@ -311,11 +349,184 @@ void printGreeting() {
 	 << " (to view the manual, type help)" << endl;
 }
 
+
 // Add a semi-colon terminator, if one is not there.
 string terminateLine(const string line) {
   return (!line.empty() && *(line.rbegin())!=';') ? (line+";") : line;
 }
 
+// cleanLine changes a C++ style comment (//) into a C-style comment (/* */) so
+// that comments don't absorb subsequent lines of code when multiline input is
+// collapsed to a single line in the history.
+//
+// ex.  if (x==1) // test x
+//        x=2;
+// becomes
+//      if (x==1) /* test x */ x=2    (all on one line)
+//
+// cleanLine is a mess because we have to check that the // is not in a string
+// or c-style comment, which entails re-inventing much of the lexer.  The
+// routine handles most cases, but multiline strings lose their newlines when
+// recorded in the history.
+typedef string::size_type size_type;
+const size_type npos=string::npos;
+
+inline size_type min(size_type a, size_type b) {
+  return a < b ? a : b;
+}
+
+// If start is an offset somewhere within a string, this returns the first
+// offset outside of the string. sym is the character used to start the string,
+// ' or ".
+size_type endOfString(const char sym, const string line, size_type start) {
+  size_type endString=line.find(sym, start);
+  if (endString == npos)
+    return npos;
+
+  size_type escapeInString=min(line.find(string("\\")+sym, start),
+                               line.find("\\\\", start));
+  if (endString < escapeInString)
+    return endString+1;
+  else
+    return endOfString(sym, line, escapeInString+2);
+}
+
+// If start is an offset somewhere within a C-style comment, this returns the
+// first offset outside of the comment.
+size_type endOfComment(const string line, size_type start) {
+  size_type endComment=line.find("*/", start);
+  if (endComment == npos)
+    return npos;
+  else
+    return endComment+2;
+}
+
+// Find the start of a string literal in the line.
+size_type stringPos(const string line, size_type start) {
+  if (start == npos)
+    return npos;
+
+  size_type pos=line.find_first_of("\'\"", start);
+  if (pos == npos)
+    return npos;
+
+  // Skip over comments /*  */ and ignore anything after //
+  size_type startComment=line.find("/*", start);
+  size_type startLineComment=line.find("//", start);
+
+  if (min(startComment,startLineComment) < pos)
+    return stringPos(line,
+                     startComment < startLineComment ?
+                         endOfComment(line, startComment+2) :
+                         npos);
+  else
+    // Nothing to skip over - the symbol actually starts a string.
+    return pos;
+}
+
+// A multiline string should retain its newlines when collapsed to a single
+// line.  This converts
+//   'hello
+//   there'
+// to
+//   'hello\nthere'
+// and
+//   "hello
+//   there"
+// to
+//   "hello" '\n' "there"
+// If the line doesn't end mid-string, this adds a space to the end to preserve
+// whitespace, since it is the last function to touch the line.
+string endString(const string line, size_type start) {
+  assert(start!=npos);
+
+  size_type pos=stringPos(line, start);
+
+  if (pos==npos)
+    // String ends in normal code.
+    return line+" ";
+  else {
+    char sym=line[pos];
+    size_type eos=endOfString(sym, line, pos+1);
+    if (eos==npos) {
+      // Line ends while in a string, attach a newline symbol.
+      switch (line[pos]) {
+        case '\'':
+          return line+"\\n";
+        case '\"':
+          return line+"\" \'\\n\' \"";
+        default:
+           assert(False);
+           return line;
+      }
+    }
+    else {
+      return endString(line, eos+1);
+    }
+  }
+}
+
+// Find the first // that isn't in a C-style comment or a string.
+size_type slashPos(const string line, size_type start) {
+  if (start == npos)
+    return npos;
+
+  size_type pos=line.find("//", start);
+  if (pos == npos)
+    return npos;
+
+  // Skip over comments /*  */ and strings both "   " and '   '
+  size_type startComment=line.find("/*", start);
+  size_type startString=line.find_first_of("\'\"", start);
+
+  if (min(startComment,startString) < pos)
+    return slashPos(line,
+                    startComment < startString ?
+                        endOfComment(line, startComment+2) :
+                        endOfString(line[startString], line, startString+1));
+  else
+    // Nothing to skip over - the // actually starts a comment.
+    return pos;
+}
+
+string endCommentOrString(const string line) {
+  size_type pos=slashPos(line, 0);
+  if (pos == npos)
+    return endString(line, 0);
+  else {
+    string sub=line;
+
+    // Replace the first // by /*
+    sub[pos+1]='*';
+
+    // Replace any */ in the comment by *!
+    while ((pos = line.find("*/", pos+2)) != npos)
+      sub[pos+1]='!';
+
+    // Tack on a */ at the end.
+    sub.append(" */ ");
+    return sub;
+  }
+}
+
+bool isSlashed(const string line) {
+  // NOTE: This doesn't fully handle escaped slashed in a string literal.
+  return line[line.size()-1]=='\\';
+}
+string deslash(const string line) {
+  return isSlashed(line) ? line.substr(0,line.size()-1) : line;
+}
+
+// This transforms a line in to the history, so that when more code is added
+// after it, the code behaves the same as if there was a newline between the
+// two lines.  This involves changing // style comments to /* */ style comments,
+// and adding explicit newlines to multiline strings.
+string cleanLine(const string line) {
+  // First remove a trailing slash, if there is one.
+  return endCommentOrString(deslash(line));
+}
+
+
 class iprompt : public icore {
   // Flag that is set to false to signal the prompt to exit.
   bool running;
@@ -325,8 +536,16 @@ class iprompt : public icore {
 
   // Code ran at start-up.
   string startline;
-  //block *startcode;
 
+  // Used for handling quit abbreviation (q).
+  protoenv *einteractive;
+  
+  void postRun(coenv &, istack &) {
+    run::cleanup();
+    camp::TeXpipepreamble=TeXpipepreamble_save;
+    camp::TeXpreamble=TeXpreamble_save;
+  }
+  
   // Commands are chopped into the starting word and the rest of the line.
   struct commandLine {
     string line;
@@ -393,11 +612,15 @@ class iprompt : public icore {
     else return false;
   }
 
+  bool q(commandLine cl) {
+    if(einteractive->ve.getType(symbol::trans("q"))) return false;
+    return quit(cl);
+  }
+
   bool reset(commandLine cl) {
     if (cl.simple()) {
       running=false;
       restart=true;
-      //startcode=0;
       startline="";
 
       uptodate=true;
@@ -419,18 +642,10 @@ class iprompt : public icore {
 
   bool input(commandLine cl) {
     string prefix="erase(); include ";
-    string line=terminateLine(prefix+cl.rest);
-    //block *code=parser::parseString(line, "<input>");
-    //if (!em->errors() && code) {
-//    if (true) {
-      running=false;
-      restart=true;
-      //startcode=code;
-      startline=line;
-//    } else {
-//      em->sync();
-//      em->clear();
-//    }
+    string line=prefix+cl.rest;
+    running=false;
+    restart=true;
+    startline=line;
     return true;
   }
 
@@ -441,7 +656,7 @@ class iprompt : public icore {
     // keywords.pl looks for ADDCOMMAND to identify special commands in the
     // auto-completion.
     ADDCOMMAND(quit,quit);
-    ADDCOMMAND(q,quit);
+    ADDCOMMAND(q,q);
     ADDCOMMAND(exit,quit);
     ADDCOMMAND(reset,reset);
     ADDCOMMAND(help, help);
@@ -466,17 +681,68 @@ class iprompt : public icore {
       return false;
   }
 
+  void addToHistory(string line) {
+    interact::addToHistory(line);
+  }
+
+  void addToLastLine(string line) {
+    // Here we clean a line at the last possible point, when we know that more
+    // code is going to be appended to it.
+    string last=interact::getLastHistoryLine();
+    interact::setLastHistoryLine(cleanLine(last)+line);
+  }
+
+  // Get input from the interactive prompt.  Such input may be over several
+  // user-typed lines if he/she ends a line a with backslash to continue input
+  // on the next line.  If continuation is true, the input started on a previous
+  // line and is being continued (either because of a backslash or the parser
+  // detecting it in multiline mode).
+  string getline(bool continuation) {
+    string prompt=getSetting<string>(continuation ? "prompt2" : "prompt");
+    string line=interact::simpleline(prompt);
+
+    if (continuation)
+      addToLastLine(line);
+    else
+      addToHistory(line);
+
+    // If the line ends in a slash, get more input.
+    return isSlashed(line) ? line+"\n"+getline(true) :
+                             line;
+  }
+
+  // Continue taking input for a line until it properly parses, or a syntax
+  // error occurs.  Returns the parsed code on success, and throws a
+  // handled_error exception on failure.
+  block *parseExtendableLine(string line) {
+    block *code=parser::parseString(line, "-", true);
+    if (code) {
+      return code;
+    } else {
+      string nextline=getline(true);
+      return parseExtendableLine(line+"\n"+nextline);
+    }
+  }
+
   void runLine(coenv &e, istack &s, string line) {
     try {
-      istring i(terminateLine(line), "-");
-      i.run(e,s);
+      if (getSetting<bool>("multiline")) {
+        block *code=parseExtendableLine(line);
+        
+        icode i(code);
+        i.run(e,s,TRANS_INTERACTIVE);
+      }
+      else {
+        istring i(terminateLine(line), "-");
+        i.run(e,s,TRANS_INTERACTIVE);
+      }
 
       if(!uptodate)
         run::updateFunction(&s);
 
-    } catch (handled_error) {
+    } catch(handled_error) {
       vm::indebugger=false;
-    } catch (interrupted&) {
+    } catch(interrupted&) {
       // Turn off the interrupted flag.
       if (em)
         em->Interrupt(false);
@@ -490,19 +756,11 @@ class iprompt : public icore {
   }
 
   void runStartCode(coenv &e, istack &s) {
-#if 0
-    if (startcode) {
-      try {
-        icode(startcode).run(e,s);
-      } catch (handled_error) {}
-    }
-#endif
     if (!startline.empty())
       runLine(e, s, startline);
   }
 
 public:
-  //iprompt() : running(false), restart(false), startcode(0) {
   iprompt() : running(false), restart(false), startline("") {
     initCommands();
   }
@@ -511,15 +769,16 @@ public:
 
   void doList() {}
 
-  void run(coenv &e, istack &s) {
+  void run(coenv &e, istack &s, transMode=TRANS_NORMAL) {
     running=true;
     interact::setCompleter(new trans::envCompleter(e.e));
+    einteractive=&(e.e);
 
     runStartCode(e, s);
 
     while (running) {
       // Read a line from the prompt.
-      string line=interact::simpleline();
+      string line=getline(false);
 
       // Copied from the old interactive prompt.  Needs to be moved.
       ShipoutNumber=0;
@@ -535,10 +794,14 @@ public:
   void process() {
     printGreeting();
     interact::init_interactive();
+    try {
+      setPath("",true);
+    } catch(handled_error) {
+    }
 
     do {
       try {
-	init();
+	init(false);
 	restart=false;
 	icore::process();
       } catch(interrupted&) {
@@ -553,7 +816,7 @@ public:
 void processCode(absyntax::block *code) {
   icode(code).process();
 }
-void processFile(const mem::string& filename) {
+void processFile(const string& filename) {
   ifile(filename).process();
 }
 void processPrompt() {
@@ -563,10 +826,10 @@ void processPrompt() {
 void runCode(absyntax::block *code) {
   icode(code).doRun();
 }
-void runString(const mem::string& s) {
+void runString(const string& s) {
   istring(s).doRun();
 }
-void runFile(const mem::string& filename) {
+void runFile(const string& filename) {
   ifile(filename).doRun();
 }
 void runPrompt() {
@@ -576,7 +839,7 @@ void runPrompt() {
 void runCodeEmbedded(absyntax::block *code, trans::coenv &e, istack &s) {
   icode(code).run(e,s);
 }
-void runStringEmbedded(const mem::string& str, trans::coenv &e, istack &s) {
+void runStringEmbedded(const string& str, trans::coenv &e, istack &s) {
   istring(str).run(e,s);
 }
 void runPromptEmbedded(trans::coenv &e, istack &s) {
