@@ -19,7 +19,7 @@ using std::ofstream;
 using std::setw;
 using vm::array;
 using vm::read;
-  
+
 namespace camp {
 
 void checkColorSpace(ColorSpace colorspace)
@@ -39,10 +39,10 @@ void checkColorSpace(ColorSpace colorspace)
 psfile::psfile(const string& filename, bool pdfformat)
   : filename(filename), pdfformat(pdfformat), pdf(false), out(NULL)
 {
-  if(filename.empty()) out=&std::cout;
+  if(filename.empty()) out=&cout;
   else out=new ofstream(filename.c_str());
   if(!out || !*out) {
-    std::cerr << "Cannot write to " << filename << std::endl;
+    cerr << "Cannot write to " << filename << endl;
     throw handled_error();
   }
 }
@@ -53,7 +53,7 @@ void psfile::close()
     out->flush();
     if(!filename.empty()) {
       if(!out->good()) {
-	std::ostringstream msg;
+	ostringstream msg;
 	msg << "Cannot write to " << filename;
 	reportError(msg);
       }
@@ -85,7 +85,6 @@ void psfile::prologue(const bbox& box)
   out->fill(prev);
 
   *out << "%%Pages: 1" << newl;
-  *out << "%%EndProlog" << newl;
   *out << "%%Page: 1 1" << newl;
 }
 
@@ -193,18 +192,18 @@ void psfile::write(path p, bool newPath)
   }    
 }
 
-// Lattice shading
-void psfile::shade(array *a, const bbox& b)
+static const char *inconsistent="inconsistent colorspaces";
+  
+void psfile::latticeshade(array *a, const bbox& b)
 {
   size_t n=a->size();
   if(n == 0) return;
   
   array *a0=read<array *>(a,0);
   size_t m=a0->size();
-
-  pen *p=read<pen *>(a0,0);
-  p->convert();
-  ColorSpace colorspace=p->colorspace();
+  setfirstpen(a0);
+  
+  ColorSpace colorspace=maxcolorspace2(a);
   checkColorSpace(colorspace);
   
   unsigned ncomponents=ColorComponents[colorspace];
@@ -219,12 +218,13 @@ void psfile::shade(array *a, const bbox& b)
        << "<< /FunctionType 0" << newl
        << "/Order 1" << newl
        << "/Domain [0 1 0 1]" << newl
-       << "/Range [0 1 0 1 0 1]" << newl
+       << "/Range [" << newl;
+  for(unsigned i=0; i < ncomponents; ++i)
+    *out << "0 1 ";
+  *out << "]" << newl
        << "/Decode [";
-  
   for(unsigned i=0; i < ncomponents; ++i)
     *out << "0 1 ";
-  
   *out << "]" << newl;
   *out << "/BitsPerSample 8" << newl;
   *out << "/Size [" << m << " " << n << "]" << newl
@@ -237,8 +237,8 @@ void psfile::shade(array *a, const bbox& b)
     for(size_t j=0; j < m; j++) {
 	pen *p=read<pen *>(ai,j);
 	p->convert();
-	if(p->colorspace() != colorspace)
-	  reportError("inconsistent shading colorspaces");
+	if(!p->promote(colorspace))
+	  reportError(inconsistent);
 	writeHex(p,ncomponents);
       }
     }
@@ -249,10 +249,11 @@ void psfile::shade(array *a, const bbox& b)
 }
 
 // Axial and radial shading
-void psfile::shade(bool axial, const ColorSpace &colorspace,
-		   const pen& pena, const pair& a, double ra,
-		   const pen& penb, const pair& b, double rb)
+void psfile::gradientshade(bool axial, const ColorSpace &colorspace,
+			   const pen& pena, const pair& a, double ra,
+			   const pen& penb, const pair& b, double rb)
 {
+  setpen(pena);
   checkColorSpace(colorspace);
   
   *out << "<< /ShadingType " << (axial ? "2" : "3") << newl
@@ -279,16 +280,13 @@ void psfile::shade(bool axial, const ColorSpace &colorspace,
        << "shfill" << newl;
 }
   
-// Gouraud shading
-void psfile::shade(array *pens, array *vertices, array *edges)
+void psfile::gouraudshade(array *pens, array *vertices, array *edges)
 {
   size_t size=pens->size();
   if(size == 0) return;
   
-  pen *p=read<pen *>(pens,0);
-  p->convert();
-  ColorSpace colorspace=p->colorspace();
-  checkColorSpace(colorspace);
+  setfirstpen(pens);
+  ColorSpace colorspace=maxcolorspace(pens);
 
   *out << "<< /ShadingType 4" << newl
        << "/ColorSpace /Device" << ColorDeviceSuffix[colorspace] << newl
@@ -298,8 +296,8 @@ void psfile::shade(array *pens, array *vertices, array *edges)
     write(read<pair>(vertices,i));
     pen *p=read<pen *>(pens,i);
     p->convert();
-    if(p->colorspace() != colorspace)
-      reportError("inconsistent shading colorspaces");
+    if(!p->promote(colorspace))
+      reportError(inconsistent);
     *out << " ";
     write(*p);
     *out << newl;
@@ -309,6 +307,73 @@ void psfile::shade(array *pens, array *vertices, array *edges)
        << "shfill" << newl;
 }
  
+// Tensor-product patch shading
+void psfile::tensorshade(array *pens, array *boundaries, array *z)
+{
+  size_t size=pens->size();
+  if(size == 0) return;
+  size_t nz=z->size();
+  
+  array *p0=read<array *>(pens,0);
+  if(checkArray(p0) != 4)
+    reportError("4 pens required");
+  setfirstpen(p0);
+  
+  ColorSpace colorspace=maxcolorspace2(pens);
+  checkColorSpace(colorspace);
+
+  *out << "<< /ShadingType 7" << newl
+       << "/ColorSpace /Device" << ColorDeviceSuffix[colorspace] << newl
+       << "/DataSource [" << newl;
+  
+  for(size_t i=0; i < size; i++) {
+    // Only edge flag 0 (new patch) is implemented since the 32% data
+    // compression (for RGB) afforded by other edge flags really isn't worth
+    // the trouble or confusion for the user. 
+    write(0);
+    path g=read<path>(boundaries,i);
+    if(!(g.cyclic() && g.size() == 4))
+      reportError("specify cyclic path of length 4");
+    for(int j=0; j < 4; ++j) {
+      write(g.point(j));
+      write(g.postcontrol(j));
+      write(g.precontrol(j+1));
+    }
+    if(nz == 0) { // Coons patch
+      static double nineth=1.0/9.0;
+      for(int j=0; j < 4; ++j) {
+	write(nineth*(-4.0*g.point(j)+6.0*(g.precontrol(j)+g.postcontrol(j))
+		      -2.0*(g.point(j-1)+g.point(j+1))
+		      +3.0*(g.precontrol(j-1)+g.postcontrol(j+1))
+		      -g.point(j+2)));
+      }
+    } else {
+      array *zi=read<array *>(z,i);
+      if(checkArray(zi) != 4)
+	reportError("specify 4 internal control points for each path");
+      for(int j=0; j < 4; ++j)
+	write(read<pair>(zi,j));
+    }
+    
+    array *pi=read<array *>(pens,i);
+    if(checkArray(pi) != 4)
+      reportError("specify 4 pens for each path");
+    for(int j=0; j < 4; ++j) {
+      pen *p=read<pen *>(pi,j);
+      p->convert();
+      if(!p->promote(colorspace))
+	reportError(inconsistent);
+      *out << " ";
+      write(*p);
+    }
+    *out << newl;
+  }
+  
+  *out << "]" << newl
+       << ">>" << newl
+       << "shfill" << newl;
+}
+ 
 inline unsigned int byte(double r) // Map [0,1] to [0,255]
 {
   if(r < 0.0) r=0.0;
@@ -318,7 +383,8 @@ inline unsigned int byte(double r) // Map [0,1] to [0,255]
   return a;
 }
 
-void psfile::writeHex(pen *p, int ncomponents) {
+void psfile::writeHex(pen *p, int ncomponents) 
+{
   switch(ncomponents) {
   case 0:
     break;
@@ -365,22 +431,21 @@ void psfile::imageheader(double width, double height, ColorSpace colorspace)
        << "image" << newl;
 }
 
-static const char *inconsistent="inconsistent colorspaces in palette";
-
 void psfile::image(array *a, array *P)
 {
   size_t asize=a->size();
   size_t Psize=P->size();
-  
   if(asize == 0 || Psize == 0) return;
   
   array *a0=read<array *>(a,0);
   size_t a0size=a0->size();
+  if(a0size == 0) return;
   
-  pen *p=read<pen *>(P,0);
-  p->convert();
-  ColorSpace colorspace=p->colorspace();
+  setfirstpen(P);
+  
+  ColorSpace colorspace=maxcolorspace(P);
   checkColorSpace(colorspace);
+  
   unsigned ncomponents=ColorComponents[colorspace];
   
   imageheader(a0size,asize,colorspace);
@@ -402,12 +467,11 @@ void psfile::image(array *a, array *P)
     array *ai=read<array *>(a,i);
     for(size_t j=0; j < a0size; j++) {
       double val=read<double>(ai,j);
-      pen *p=read<pen *>(P,(int) ((val-min)*step+0.5));
+      size_t index=(size_t) ((val-min)*step+0.5);
+      pen *p=read<pen *>(P,index < Psize ? index : Psize-1);
       p->convert();
-      
-      if(p->colorspace() != colorspace)
+      if(!p->promote(colorspace))
 	reportError(inconsistent);
-
       writeHex(p,ncomponents);
     }
   }
@@ -418,16 +482,17 @@ void psfile::image(array *a, array *P)
 void psfile::image(array *a)
 {
   size_t asize=a->size();
-  
   if(asize == 0) return;
   
   array *a0=read<array *>(a,0);
   size_t a0size=a0->size();
+  if(a0size == 0) return;
   
-  pen *p=read<pen *>(a0,0);
-  p->convert();
-  ColorSpace colorspace=p->colorspace();
+  setfirstpen(a0);
+  
+  ColorSpace colorspace=maxcolorspace2(a);
   checkColorSpace(colorspace);
+  
   unsigned ncomponents=ColorComponents[colorspace];
   
   imageheader(a0size,asize,colorspace);
@@ -437,10 +502,8 @@ void psfile::image(array *a)
     for(size_t j=0; j < a0size; j++) {
       pen *p=read<pen *>(ai,j);
       p->convert();
-
-      if(p->colorspace() != colorspace)
+      if(!p->promote(colorspace))
 	reportError(inconsistent);
-
       writeHex(p,ncomponents);
     }
   }
