@@ -10,7 +10,6 @@
 #define PSFILE_H
 
 #include <fstream>
-#include <stack>
 #include <iomanip>
 #include <sstream>
 
@@ -54,7 +53,7 @@ protected:
   
 public: 
   psfile(const string& filename, bool pdfformat);
-  psfile() {pdf=settings::pdf(settings::getSetting<mem::string>("tex"));}
+  psfile() {pdf=settings::pdf(settings::getSetting<string>("tex"));}
 
   ~psfile();
   
@@ -142,14 +141,15 @@ public:
     }
   }
   
-  void shade(vm::array *a, const bbox& b);
+  void latticeshade(vm::array *a, const bbox& b);
   
-  void shade(bool axial, const ColorSpace& colorspace,
-	     const pen& pena, const pair& a, double ra,
-	     const pen& penb, const pair& b, double rb);
-  
-  void shade(vm::array *pens, vm::array *vertices, vm::array *edges);
+  void gradientshade(bool axial, const ColorSpace& colorspace,
+		     const pen& pena, const pair& a, double ra,
+		     const pen& penb, const pair& b, double rb);
   
+  void gouraudshade(vm::array *pens, vm::array *vertices, vm::array *edges);
+  void tensorshade(vm::array *pens, vm::array *boundaries, vm::array *z);
+
   void imageheader(double width, double height, ColorSpace colorspace);
   
   void image(vm::array *a, vm::array *p);
@@ -194,6 +194,34 @@ public:
     *out << s;
   }
 
+  // Determine shading and image transparency based on first pen.
+  void setfirstpen(vm::array *pens) {
+    if(pens->size() > 0) {
+      pen *p=vm::read<pen *>(pens,0);
+      setpen(*p);
+    }
+  }
+  
+  ColorSpace maxcolorspace(vm::array *pens) {
+    int colorspace=0;
+    size_t size=pens->size();
+    for(size_t i=0; i < size; i++) {
+      pen *p=vm::read<pen *>(pens,i);
+      p->convert();
+      colorspace=max(colorspace,(int) p->colorspace());
+    }
+    return (ColorSpace) colorspace;
+  }
+  
+  ColorSpace maxcolorspace2(vm::array *penarray) {
+    int colorspace=0;
+    size_t size=penarray->size();
+    for(size_t i=0; i < size; i++)
+      colorspace=max(colorspace,
+		     (int) maxcolorspace(vm::read<vm::array*>(penarray,i)));
+    return (ColorSpace) colorspace;
+  }
+
 };
 
 } //namespace camp
