@@ -9,7 +9,6 @@
 #include <iomanip>
 #include <cstdlib>
 #include <cerrno>
-#include <vector>
 #include <sys/stat.h>
 #include <cfloat>
 #include <locale.h>
@@ -30,67 +29,141 @@
 #include "interact.h"
 #include "locate.h"
 #include "lexical.h"
-#include "memory.h"
 #include "record.h"
 #include "env.h"
 #include "item.h"
 #include "refaccess.h"
 
 #ifdef HAVE_LIBCURSES
+extern "C" {
 #include <curses.h>
 
+// Workaround broken curses.h files:
+#ifdef clear
+#undef clear
+#endif
+
 // Workaround missing symbolic link under CYGWIN
 #if defined(__CYGWIN__) && defined(HAVE_LIBNCURSES)
 #include <ncurses/term.h>
 #else
 #include <term.h>
 #endif
+}
 #endif
 
-using std::vector;
 using vm::item;
 
 using trans::itemRefAccess;
 using trans::refAccess;
 using trans::varEntry;
-  
+
 void doConfig(string filename);
 
 namespace settings {
   
 using camp::pair;
   
-#ifdef __CYGWIN__
-const bool msdos=true;
-const char *HOME="USERPROFILE";
-const char pathSeparator=';';
-const string defaultPSViewer=
-  "c:\\Program Files\\Ghostgum\\gsview\\gsview32.exe";
-const string defaultPDFViewer=
-  "c:\\Program Files\\Adobe\\Acrobat 7.0\\Reader\\AcroRd32.exe";
-const string defaultGhostscript=
-  "c:\\Program Files\\gs\\gs8.54\\bin\\gswin32c.exe";
-const string defaultPython="c:\\Python24\\python.exe";
-const string defaultDisplay="imdisplay";
-#undef ASYMPTOTE_SYSDIR
-#define ASYMPTOTE_SYSDIR "c:\\Program Files\\Asymptote"
-const string docdir=".";
-#else  
-const bool msdos=false;
+string asyInstallDir; // Used only by msdos
+string defaultXasy="xasy";
+
+#ifndef __CYGWIN__
+  
+bool msdos=false;
 const char *HOME="HOME";
 const char pathSeparator=':';
-const string defaultPSViewer="gv";
+string defaultPSViewer="gv";
 #ifdef __APPLE__
-const string defaultPDFViewer="open";
+string defaultPDFViewer="open";
 #else  
-const string defaultPDFViewer="acroread";
+string defaultPDFViewer="acroread";
 #endif  
-const string defaultGhostscript="gs";
-const string defaultDisplay="display";
-const string defaultPython="";
+string defaultGhostscript="gs";
+string defaultDisplay="display";
+string defaultPython;
 const string docdir=ASYMPTOTE_DOCDIR;
+void queryRegistry() {}
+  
+#else  
+  
+bool msdos=true;
+const char *HOME="USERPROFILE";
+const char pathSeparator=';';
+string defaultPSViewer="gsview32.exe";
+string defaultPDFViewer="AcroRd32.exe";
+string defaultGhostscript="gswin32c.exe";
+string defaultPython="python.exe";
+string defaultDisplay="imdisplay";
+#undef ASYMPTOTE_SYSDIR
+#define ASYMPTOTE_SYSDIR asyInstallDir
+const string docdir=".";
+  
+#include <dirent.h>
+  
+// Use key to look up an entry in the MSWindows registry, respecting wild cards
+string getEntry(const string& key)
+{
+  string path="/proc/registry/HKEY_LOCAL_MACHINE/SOFTWARE/"+key;
+  size_t star;
+  string head;
+  while((star=path.find("*")) < string::npos) {
+    string prefix=path.substr(0,star);
+    string suffix=path.substr(star+1);
+    size_t slash=suffix.find("/");
+    if(slash < string::npos) {
+      path=suffix.substr(slash);
+      suffix=suffix.substr(0,slash);
+    } else {
+      path=suffix;
+      suffix="";
+    }
+    string directory=head+stripFile(prefix);
+    string file=stripDir(prefix);
+    DIR *dir=opendir(directory.c_str());
+    if(dir == NULL) return "";
+    dirent *p;
+    string rsuffix=suffix;
+    reverse(rsuffix.begin(),rsuffix.end());
+    while((p=readdir(dir)) != NULL) {
+      string dname=p->d_name;
+      string rdname=dname;
+      reverse(rdname.begin(),rdname.end());
+      if(dname != "." && dname != ".." && 
+	 dname.substr(0,file.size()) == file &&
+	 rdname.substr(0,suffix.size()) == rsuffix) {
+	head=directory+p->d_name;
+	break;
+      }
+    }
+    if(p == NULL) return "";
+  }
+  std::ifstream fin((head+path).c_str());
+  if(fin) {
+    string s;
+    getline(fin,s);
+    size_t end=s.find('\0');
+    if(end < string::npos)
+      return s.substr(0,end);
+  }
+  return "";
+}
+  
+void queryRegistry()
+{
+  string gs=getEntry("AFPL Ghostscript/*/GS_DLL");
+  if(gs.empty())
+    gs=getEntry("GPL Ghostscript/*/GS_DLL");
+  defaultGhostscript=stripFile(gs)+defaultGhostscript;
+  defaultPDFViewer=getEntry("Adobe/Acrobat Reader/*/InstallPath/@")+"\\"+
+    defaultPDFViewer;
+  defaultPSViewer=getEntry("Ghostgum/GSview/*")+"\\gsview\\"+defaultPSViewer;
+  defaultPython=getEntry("Python/PythonCore/*/InstallPath/@")+defaultPython;
+  asyInstallDir=getEntry("Microsoft/Windows/CurrentVersion/Uninstall/Asymptote/InstallLocation");
+  defaultXasy=asyInstallDir+"\\"+defaultXasy;
+}
+  
 #endif  
-
+  
 const char PROGRAM[]=PACKAGE_NAME;
 const char VERSION[]=PACKAGE_VERSION;
 const char BUGREPORT[]=PACKAGE_BUGREPORT;
@@ -100,19 +173,28 @@ char *argv0;
 
 // The verbosity setting, a global variable.
 int verbose;
-int safe=1;
   
-// Work around backwards-incompatible command-line options of gv-3.6.1.
-string gvOptionPrefix="-";
+// Colorspace conversion flags (stored in global variables for efficiency). 
+bool gray;
+bool bw;  
+bool rgb;
+bool cmyk;
+  
+// Disable system calls.
+bool safe=true;
+// Enable writing to (or changing to) other directories
+bool globaloption=false;
+bool globaloutname=false;
+  
+bool global() {return globaloption || !safe;}
   
 int ShipoutNumber=0;
   
 const string suffix="asy";
 const string guisuffix="gui";
   
-bool TeXinitialized=false;
 string initdir;
-mem::string historyname;
+string historyname;
 
 camp::pen *initialdefaultpen=NULL;
 camp::pen defaultpen=camp::pen::startupdefaultpen();
@@ -131,32 +213,32 @@ types::record *getSettingsModule() {
 
 // The dictionaries of long options and short options.
 class option;
-typedef mem::map<CONST mem::string, option *> optionsMap_t;
+typedef mem::map<CONST string, option *> optionsMap_t;
 optionsMap_t optionsMap;
 typedef mem::map<CONST char, option *> codeMap_t;
 codeMap_t codeMap;
   
 struct option : public gc {
-  mem::string name;
+  string name;
   char code;      // Command line option, i.e. 'V' for -V.
   bool argument;  // If it takes an argument on the command line.  This is set
                   // based on whether argname is empty.
-  mem::string argname; // The argument name for printing the description.
-  mem::string desc; // One line description of what the option does.
+  string argname; // The argument name for printing the description.
+  string desc; // One line description of what the option does.
   bool cmdlineonly; // If it is only available on the command line.
-  mem::string Default; // A string containing an optional default value.
+  string Default; // A string containing an optional default value.
 
-  option(mem::string name, char code, mem::string argname, mem::string desc,
-	 bool cmdlineonly=false, mem::string Default="")
+  option(string name, char code, string argname, string desc,
+	 bool cmdlineonly=false, string Default="")
     : name(name), code(code), argument(!argname.empty()), argname(argname),
       desc(desc), cmdlineonly(cmdlineonly), Default(Default) {}
 
   virtual ~option() {}
 
   // Builds this option's contribution to the optstring argument of get_opt().
-  virtual mem::string optstring() {
+  virtual string optstring() {
     if (code) {
-      mem::string base;
+      string base;
       base.push_back(code);
       if(argument) base.push_back(':');
       return base;
@@ -191,7 +273,7 @@ struct option : public gc {
   }
 
   // The "-f,-outformat format" part of the option.
-  virtual mem::string describeStart() {
+  virtual string describeStart() {
     ostringstream ss;
     if (code)
       ss << "-" << code << ",";
@@ -206,7 +288,7 @@ struct option : public gc {
     // Don't show the option if it has no desciption.
     if (!desc.empty()) {
       const unsigned int WIDTH=22;
-      mem::string start=describeStart();
+      string start=describeStart();
       cerr << std::left << std::setw(WIDTH) << start;
       if (start.size() >= WIDTH) {
         cerr << endl;
@@ -214,22 +296,26 @@ struct option : public gc {
       }
       cerr << desc;
       if(cmdlineonly) cerr << "; command-line only";
-      if(Default != "") cerr << " [" << Default << "]";
+      if(Default != "")
+	cerr << " [" << Default << "]";
       cerr << endl;
     }
   }
+  
+  virtual void reset() {
+  }
 };
 
-const mem::string noarg;
+const string noarg;
 
 struct setting : public option {
   types::ty *t;
 
-  setting(mem::string name, char code, mem::string argname, mem::string desc,
-          types::ty *t, mem::string Default)
+  setting(string name, char code, string argname, string desc,
+          types::ty *t, string Default)
     : option(name, code, argname, desc, false,Default), t(t) {}
 
-  virtual void reset() = 0;
+  void reset() = 0;
 
   virtual trans::access *buildAccess() = 0;
 
@@ -237,26 +323,17 @@ struct setting : public option {
   virtual void add() {
     option::add();
 
-#if 0
-    settingsModule->e.addVar(symbol::trans(name), buildVarEntry());
-#endif
     settingsModule->add(name, t, buildAccess());
   }
-
-#if 0
-  varEntry *buildVarEntry() {
-    return new varEntry(t, buildAccess(), getSettingsModule());
-  }
-#endif
 };
 
 struct itemSetting : public setting {
   item defaultValue;
   item value;
 
-  itemSetting(mem::string name, char code,
-              mem::string argname, mem::string desc,
-              types::ty *t, item defaultValue, mem::string Default="")
+  itemSetting(string name, char code,
+              string argname, string desc,
+              types::ty *t, item defaultValue, string Default="")
     : setting(name, code, argname, desc, t, Default),
       defaultValue(defaultValue) {reset();}
 
@@ -279,7 +356,7 @@ item& Setting(string name) {
 }
   
 struct boolSetting : public itemSetting {
-  boolSetting(mem::string name, char code, mem::string desc,
+  boolSetting(string name, char code, string desc,
               bool defaultValue=false)
     : itemSetting(name, code, noarg, desc,
                   types::primBoolean(), (item)defaultValue,
@@ -290,11 +367,11 @@ struct boolSetting : public itemSetting {
     return true;
   }
 
-  option *negation(mem::string name) {
+  option *negation(string name) {
     struct negOption : public option {
       boolSetting &base;
 
-      negOption(boolSetting &base, mem::string name)
+      negOption(boolSetting &base, string name)
         : option(name, 0, noarg, ""), base(base) {}
 
       bool getOption() {
@@ -309,7 +386,7 @@ struct boolSetting : public itemSetting {
     setting::add();
     negation("no"+name)->add();
     if (code) {
-      mem::string nocode="no"; nocode.push_back(code);
+      string nocode="no"; nocode.push_back(code);
       negation(nocode)->add();
     }
   }
@@ -319,7 +396,7 @@ struct boolSetting : public itemSetting {
   struct multiOption : public option {
     typedef mem::list<boolSetting *> setlist;
     setlist set;
-    multiOption(mem::string name, char code, mem::string desc)
+    multiOption(string name, char code, string desc)
       : option(name, code, noarg, desc, true) {}
 
     void add(boolSetting *s) {
@@ -336,11 +413,11 @@ struct boolSetting : public itemSetting {
       return true;
     }
 
-    option *negation(mem::string name) {
+    option *negation(string name) {
       struct negOption : public option {
         multiOption &base;
 
-        negOption(multiOption &base, mem::string name)
+        negOption(multiOption &base, string name)
           : option(name, 0, noarg, ""), base(base) {}
 
         bool getOption() {
@@ -355,7 +432,7 @@ struct boolSetting : public itemSetting {
       option::add();
       negation("no"+name)->add();
       if (code) {
-        mem::string nocode="no"; nocode.push_back(code);
+        string nocode="no"; nocode.push_back(code);
         negation(nocode)->add();
       }
 
@@ -368,8 +445,8 @@ struct boolSetting : public itemSetting {
 typedef boolSetting::multiOption multiOption;
 
 struct argumentSetting : public itemSetting {
-  argumentSetting(mem::string name, char code,
-                  mem::string argname, mem::string desc,
+  argumentSetting(string name, char code,
+                  string argname, string desc,
                   types::ty *t, item defaultValue)
     : itemSetting(name, code, argname, desc, t, defaultValue) 
   {
@@ -378,49 +455,63 @@ struct argumentSetting : public itemSetting {
 };
 
 struct stringSetting : public argumentSetting {
-  stringSetting(mem::string name, char code,
-                mem::string argname, mem::string desc,
-                mem::string defaultValue)
+  stringSetting(string name, char code,
+                string argname, string desc,
+                string defaultValue)
     : argumentSetting(name, code, argname, desc,
-              types::primString(), (item)defaultValue) {}
+		      types::primString(), (item)defaultValue) {}
 
   bool getOption() {
-    value=(item)(mem::string)optarg;
+    value=(item)(string)optarg;
+    return true;
+  }
+};
+
+struct stringOutnameSetting : public argumentSetting {
+  stringOutnameSetting(string name, char code,
+		       string argname, string desc,
+		       string defaultValue)
+    : argumentSetting(name, code, argname, desc,
+		      types::primString(), (item)defaultValue) {}
+
+  bool getOption() {
+    value=(item)(string)
+      ((globaloutname || global()) ? optarg : stripDir(optarg));
     return true;
   }
 };
 
 struct userSetting : public argumentSetting {
-  userSetting(mem::string name, char code,
-	      mem::string argname, mem::string desc,
-	      mem::string defaultValue)
+  userSetting(string name, char code,
+	      string argname, string desc,
+	      string defaultValue)
     : argumentSetting(name, code, argname, desc,
 		      types::primString(), (item)defaultValue) {}
 
   bool getOption() {
-    mem::string s=vm::get<mem::string>(value)+mem::string(optarg);
+    string s=vm::get<string>(value)+string(optarg);
     s.push_back(';');
     value=(item) s;
     return true;
   }
 };
 
-mem::string GetEnv(mem::string s, mem::string Default) {
+string GetEnv(string s, string Default) {
   transform(s.begin(), s.end(), s.begin(), toupper);        
-  string t=Getenv(("ASYMPTOTE_"+s).c_str());
-  return t != "" ? mem::string(t) : Default;
+  string t=Getenv(("ASYMPTOTE_"+s).c_str(),msdos);
+  return t != "" ? string(t) : Default;
 }
   
 struct envSetting : public stringSetting {
-  envSetting(mem::string name, mem::string Default)
+  envSetting(string name, string Default)
     : stringSetting(name, 0, " ", "", GetEnv(name,Default)) {}
 };
 
 template<class T>
 struct dataSetting : public argumentSetting {
   string text;
-  dataSetting(const char *text, mem::string name, char code,
-	      mem::string argname, mem::string desc, types::ty *type,
+  dataSetting(const char *text, string name, char code,
+	      string argname, string desc, types::ty *type,
 	      T defaultValue)
     : argumentSetting(name, code, argname, desc,
 		      type, (item)defaultValue), text(text) {}
@@ -437,36 +528,36 @@ struct dataSetting : public argumentSetting {
 };
 
 struct intSetting : public dataSetting<int> {
-  intSetting(mem::string name, char code,
-	     mem::string argname, mem::string desc, int defaultValue=0)
+  intSetting(string name, char code,
+	     string argname, string desc, int defaultValue=0)
     : dataSetting<int>("an int", name, code, argname, desc,
 		       types::primInt(), defaultValue) {}
 };
   
 struct realSetting : public dataSetting<double> {
-  realSetting(mem::string name, char code,
-	     mem::string argname, mem::string desc, double defaultValue=0.0)
+  realSetting(string name, char code,
+	      string argname, string desc, double defaultValue=0.0)
     : dataSetting<double>("a real", name, code, argname, desc,
-		       types::primReal(), defaultValue) {}
+			  types::primReal(), defaultValue) {}
 };
   
 struct pairSetting : public dataSetting<pair> {
-  pairSetting(mem::string name, char code,
-	     mem::string argname, mem::string desc, pair defaultValue=0.0)
+  pairSetting(string name, char code,
+	      string argname, string desc, pair defaultValue=0.0)
     : dataSetting<pair>("a pair", name, code, argname, desc,
-		       types::primPair(), defaultValue) {}
+			types::primPair(), defaultValue) {}
 };
   
 // For setting the alignment of a figure on the page.
 struct alignSetting : public argumentSetting {
-  alignSetting(mem::string name, char code,
-                  mem::string argname, mem::string desc,
-                  int defaultValue=CENTER)
+  alignSetting(string name, char code,
+	       string argname, string desc,
+	       int defaultValue=CENTER)
     : argumentSetting(name, code, argname, desc,
-                  types::primInt(), (item)defaultValue) {}
+		      types::primInt(), (item)defaultValue) {}
 
   bool getOption() {
-    mem::string str=optarg;
+    string str=optarg;
     if (str=="C")
       value=(int)CENTER;
     else if (str=="T")
@@ -488,14 +579,26 @@ template <class T>
 struct refSetting : public setting {
   T *ref;
   T defaultValue;
+  string text;
 
-  refSetting(mem::string name, char code, mem::string argname,
-             mem::string desc, types::ty *t, T *ref, T defaultValue)
-    : setting(name, code, argname, desc, t, "0"),
-      ref(ref), defaultValue(defaultValue) {
+  refSetting(string name, char code, string argname,
+             string desc, types::ty *t, T *ref, T defaultValue,
+	     string Default, const char *text="")
+    : setting(name, code, argname, desc, t, Default),
+      ref(ref), defaultValue(defaultValue), text(text) {
     reset();
   }
 
+  virtual bool getOption() {
+    try {
+      *ref=lexical::cast<T>(optarg);
+    } catch (lexical::bad_cast&) {
+      error("option requires " + text + " as an argument");
+      return false;
+    }
+    return true;
+  }
+  
   virtual void reset() {
     *ref=defaultValue;
   }
@@ -505,10 +608,45 @@ struct refSetting : public setting {
   }
 };
 
+struct boolrefSetting : public refSetting<bool> {
+  boolrefSetting(string name, char code, string desc, bool *ref)
+    : refSetting<bool>(name, code, noarg, desc,
+		       types::primBoolean(), ref, false, "false") {}
+  bool getOption() {
+    // Increment the value.
+    *ref=true;
+    return true;
+  }
+  
+  option *negation(string name) {
+    struct negOption : public option {
+      boolrefSetting &base;
+
+      negOption(boolrefSetting &base, string name)
+        : option(name, 0, noarg, ""), base(base) {}
+
+      bool getOption() {
+        *(base.ref)=false;
+        return true;
+      }
+    };
+    return new negOption(*this, name);
+  }
+  
+  void add() {
+    setting::add();
+    negation("no"+name)->add();
+    if (code) {
+      string nocode="no"; nocode.push_back(code);
+      negation(nocode)->add();
+    }
+  }
+};
+
 struct incrementSetting : public refSetting<int> {
-  incrementSetting(mem::string name, char code, mem::string desc, int *ref)
+  incrementSetting(string name, char code, string desc, int *ref)
     : refSetting<int>(name, code, noarg, desc,
-              types::primInt(), ref, 0) {}
+		      types::primInt(), ref, 0, "0") {}
 
   bool getOption() {
     // Increment the value.
@@ -516,11 +654,11 @@ struct incrementSetting : public refSetting<int> {
     return true;
   }
   
-  option *negation(mem::string name) {
+  option *negation(string name) {
     struct negOption : public option {
       incrementSetting &base;
 
-      negOption(incrementSetting &base, mem::string name)
+      negOption(incrementSetting &base, string name)
         : option(name, 0, noarg, ""), base(base) {}
 
       bool getOption() {
@@ -530,27 +668,47 @@ struct incrementSetting : public refSetting<int> {
     };
     return new negOption(*this, name);
   }
-
+  
   void add() {
     setting::add();
     negation("no"+name)->add();
     if (code) {
-      mem::string nocode="no"; nocode.push_back(code);
+      string nocode="no"; nocode.push_back(code);
       negation(nocode)->add();
     }
   }
 };
 
+struct incrementOption : public option {
+  int *ref;
+  int level;
+  
+  incrementOption(string name, char code, string desc, int *ref,
+		  int level=1)
+    : option(name, code, noarg, desc, true), ref(ref), level(level) {}
+
+  bool getOption() {
+    // Increment the value.
+    (*ref) += level;
+    return true;
+  }
+};
+
 void addOption(option *o) {
   o->add();
 }
 
-void usage(const char *program)
+void version()
 {
   cerr << PROGRAM << " version " << VERSION
        << " [(C) 2004 Andy Hammerlindl, John C. Bowman, Tom Prince]" 
-       << endl
-       << "\t\t\t" << "http://asymptote.sourceforge.net/"
+       << endl;
+}
+
+void usage(const char *program)
+{
+  version();
+  cerr << "\t\t\t" << "http://asymptote.sourceforge.net/"
        << endl
        << "Usage: " << program << " [options] [file ...]"
        << endl;
@@ -560,7 +718,7 @@ void reportSyntax() {
   cerr << endl;
   usage(argv0);
   cerr << endl << "Type '" << argv0
-    << " -h' for a description of options." << endl;
+       << " -h' for a description of options." << endl;
   exit(1);
 }
 
@@ -576,7 +734,7 @@ void displayOptions()
 }
 
 struct helpOption : public option {
-  helpOption(mem::string name, char code, mem::string desc)
+  helpOption(string name, char code, string desc)
     : option(name, code, noarg, desc, true) {}
 
   bool getOption() {
@@ -590,23 +748,49 @@ struct helpOption : public option {
   }
 };
 
-// For security reasons, safe isn't a field of the settings module.
-struct safeOption : public option {
-  bool value;
-
-  safeOption(mem::string name, char code, mem::string desc, bool value)
-    : option(name, code, noarg, desc, true, value ? "true" : "false"),
-      value(value) {}
+struct versionOption : public option {
+  versionOption(string name, char code, string desc)
+    : option(name, code, noarg, desc, true) {}
 
   bool getOption() {
-    safe=value;
+    version();
+    exit(0);
+
+    // Unreachable code.
     return true;
   }
 };
 
+// For security reasons, these options aren't fields of the settings module.
+struct boolOption : public option {
+  bool *variable;
+  bool value;
 
-mem::string build_optstring() {
-  mem::string s;
+  boolOption(string name, char code, string desc,
+	     bool *variable, bool value, bool Default)
+    : option(name, code, noarg, desc, true, Default ? "true" : "false"),
+      variable(variable), value(value) {}
+
+  bool getOption() {
+    *variable=value;
+    return true;
+  }
+};
+
+struct stringOption : public option {
+  char **variable;
+  stringOption(string name, char code, string argname,
+	       string desc, char **variable)
+    : option(name, code, argname, desc, true), variable(variable) {}
+
+  bool getOption() {
+    *variable=optarg;
+    return true;
+  }
+};
+
+string build_optstring() {
+  string s;
   for (codeMap_t::iterator p=codeMap.begin(); p !=codeMap.end(); ++p)
     s +=p->second->optstring();
 
@@ -631,16 +815,21 @@ c_option *build_longopts() {
   return longopts;
 }
 
-void resetOptions() {
-  verbose=0;
+void resetOptions()
+{
+  for(optionsMap_t::iterator opt=optionsMap.begin(); opt != optionsMap.end();
+      ++opt)
+    if(opt->first != "config" && opt->first != "dir")
+      opt->second->reset();
 }
   
 void getOptions(int argc, char *argv[])
 {
+  globaloutname=true;
   bool syntax=false;
   optind=0;
 
-  mem::string optstring=build_optstring();
+  string optstring=build_optstring();
   //cerr << "optstring: " << optstring << endl;
   c_option *longopts=build_longopts();
   int long_index = 0;
@@ -672,6 +861,7 @@ void getOptions(int argc, char *argv[])
   
   if (syntax)
     reportSyntax();
+  globaloutname=false;
 }
 
 #ifdef USEGC
@@ -679,6 +869,8 @@ void no_GCwarn(char *, GC_word) {}
 #endif
 
 void initSettings() {
+  queryRegistry();
+
   settingsModule=new types::dummyRecord(symbol::trans("settings"));
   
   multiOption *view=new multiOption("View", 'V', "View output");
@@ -692,26 +884,36 @@ void initSettings() {
   addOption(view);
 
   addOption(new realSetting("deconstruct", 'x', "X",
-                     "Deconstruct into transparent GIF objects magnified by X",
+			    "Deconstruct into transparent objects magnified by X",
 			    0.0));
+  addOption(new stringSetting("xformat", 0, "format", 
+			      "GUI deconstruction format [\"gif\"]","gif"));
   addOption(new boolSetting("clearGUI", 'c', "Clear GUI operations"));
   addOption(new boolSetting("ignoreGUI", 'i', "Ignore GUI operations"));
   addOption(new stringSetting("outformat", 'f', "format",
-		      "Convert each output file to specified format", ""));
-  addOption(new stringSetting("outname", 'o', "name",
-			      "Alternative output name for first file",
+			      "Convert each output file to specified format",
 			      ""));
+  addOption(new stringOutnameSetting("outname", 'o', "name",
+				     "Alternative output name for first file",
+				     ""));
   addOption(new helpOption("help", 'h', "Show summary of options"));
+  addOption(new versionOption("version", 0, "Show version"));
 
   addOption(new pairSetting("offset", 'O', "pair",
 			    "PostScript offset [(0,0)]"));
   addOption(new alignSetting("align", 'a', "C|B|T|Z",
-		"Center, Bottom, Top, or Zero page alignment [Center]"));
+			     "Center, Bottom, Top, or Zero page alignment [Center]"));
   
   addOption(new boolSetting("debug", 'd', "Enable debugging messages"));
   addOption(new incrementSetting("verbose", 'v',
 				 "Increase verbosity level", &verbose));
+  // Resolve ambiguity with --version
+  addOption(new incrementOption("vv", 0,"", &verbose,2));
+  addOption(new incrementOption("novv", 0,"", &verbose,-2));
+  
   addOption(new boolSetting("keep", 'k', "Keep intermediate files"));
+  addOption(new boolSetting("keepaux", 0,
+			    "Keep intermediate LaTeX .aux files"));
   addOption(new stringSetting("tex", 0,"engine",
 			      "TeX engine (\"latex|pdflatex|tex|pdftex|none\") [\"latex\"]",
 			      "latex"));
@@ -725,6 +927,8 @@ void initSettings() {
                             "Interactive prompt auto-completion", true));
   addOption(new boolSetting("listvariables", 'l',
 			    "List available global functions and variables"));
+  addOption(new boolSetting("where", 0,
+			    "Show where listed variables are declared"));
   
   multiOption *mask=new multiOption("mask", 'm',
 				    "Mask fpu exceptions");
@@ -734,20 +938,39 @@ void initSettings() {
 			    "Mask fpu exceptions in interactive mode", true));
   addOption(mask);
 
-  addOption(new boolSetting("bw", 0,
-			    "Convert all colors to black and white"));
-  addOption(new boolSetting("gray", 0, "Convert all colors to grayscale"));
-  addOption(new boolSetting("rgb", 0, "Convert cmyk colors to rgb"));
-  addOption(new boolSetting("cmyk", 0, "Convert rgb colors to cmyk"));
+  addOption(new boolrefSetting("bw", 0,
+			       "Convert all colors to black and white",&bw));
+  addOption(new boolrefSetting("gray", 0, "Convert all colors to grayscale",
+			       &gray));
+  addOption(new boolrefSetting("rgb", 0, "Convert cmyk colors to rgb",&rgb));
+  addOption(new boolrefSetting("cmyk", 0, "Convert rgb colors to cmyk",&cmyk));
 
-  addOption(new safeOption("safe", 0,
-			   "Disable system call", true));
-  addOption(new safeOption("unsafe", 0,
-			   "Enable system call", false));
+  addOption(new boolOption("safe", 0,
+			   "Disable system call", &safe, true, true));
+  addOption(new boolOption("unsafe", 0,
+			   "Enable system call (=> global)", &safe, false,
+			   false));
+  addOption(new boolOption("global", 0,
+			   "Allow write to other directory",
+			   &globaloption, true, false));
+  addOption(new boolOption("noglobal", 0,
+			   "", &globaloption, false, true));
+  addOption(new stringOption("cd", 0, "directory", "Set current directory",
+			     &startpath));
   
   addOption(new stringSetting("prompt", 0,"string","Prompt [\"> \"]","> "));
-  addOption(new boolSetting("quiet", 'q',
-			    "Suppress welcome message"));
+  addOption(new stringSetting("prompt2", 0,"string",
+                              "Continuation prompt for multiline input "
+                              "[\"..\"]", ".."));
+  addOption(new boolSetting("multiline", 0,
+                            "Input code over multiple lines at the prompt"));
+
+  addOption(new boolSetting("quiet", 'q',"Suppress welcome message"));
+  addOption(new boolSetting("wait", 0,
+			    "Wait for child processes to finish before exiting"));
+  // Signal parent process at completion of each shipout
+  addOption(new boolSetting("signal", 0, ""));
+			    
   addOption(new boolSetting("localhistory", 0,
 			    "Use a local interactive history file"));
   addOption(new intSetting("historylines", 0, "n",
@@ -760,23 +983,28 @@ void initSettings() {
   addOption(new boolSetting("autorotate", 0,
 			    "Enable automatic PDF page rotation",
 			    false));
-  addOption(new userSetting("user", 'u', "x",
-			    "General purpose user string  [\"\"]", ""));
+  addOption(new stringSetting("autoimport", 0, "string",
+			      "Module to automatically import [\"\"]", ""));
+  addOption(new userSetting("user", 'u', "string",
+			    "General purpose user string [\"\"]", ""));
   
   addOption(new realSetting("paperwidth", 0, "bp", ""));
   addOption(new realSetting("paperheight", 0, "bp", ""));
   
+  addOption(new stringSetting("dvipsOptions", 0, "string", "", ""));
+  
   addOption(new envSetting("config","config."+suffix));
   addOption(new envSetting("pdfviewer", defaultPDFViewer));
   addOption(new envSetting("psviewer", defaultPSViewer));
   addOption(new envSetting("gs", defaultGhostscript));
   addOption(new envSetting("texpath", ""));
+  addOption(new envSetting("texcommand", ""));
   addOption(new envSetting("dvips", "dvips"));
   addOption(new envSetting("convert", "convert"));
   addOption(new envSetting("display", defaultDisplay));
   addOption(new envSetting("animate", "animate"));
   addOption(new envSetting("python", defaultPython));
-  addOption(new envSetting("xasy", "xasy"));
+  addOption(new envSetting("xasy", defaultXasy));
   addOption(new envSetting("papertype", "letter"));
   addOption(new envSetting("dir", ""));
 }
@@ -788,11 +1016,6 @@ char *getArg(int n) { return argList[n]; }
 void setInteractive() {
   if(numArgs() == 0 && !getSetting<bool>("listvariables")) {
     interact::interactive=true;
-    
-    // Work around backwards-incompatible command-line options of gv-3.6.1.
-    if(!msdos && (getSetting<mem::string>("pdfviewer") == "gv" ||
-		  getSetting<mem::string>("psviewer") == "gv"))
-      gvOptionPrefix=System("gv --version",2) == 0 ? "--" : "-";
   }
   
   historyname=getSetting<bool>("localhistory") ? "."+suffix+"_history" 
@@ -814,13 +1037,13 @@ bool trap() {
     return !getSetting<bool>("batchMask");
 }
 
-mem::string outname() {
-  mem::string name=getSetting<mem::string>("outname");
+string outname() {
+  string name=getSetting<string>("outname");
   return name.empty() ? "out" : name;
 }
 
 void initDir() {
-  initdir=Getenv(HOME,false)+"/."+suffix;
+  initdir=Getenv(HOME,msdos)+"/."+suffix;
   mkdir(initdir.c_str(),0xFFFF);
 }
   
@@ -828,7 +1051,7 @@ void setPath() {
   searchPath.clear();
   searchPath.push_back(".");
   searchPath.push_back(initdir);
-  string asydir=getSetting<mem::string>("dir");
+  string asydir=getSetting<string>("dir");
   if(asydir != "") {
     size_t p,i=0;
     while((p=asydir.find(pathSeparator,i)) < string::npos) {
@@ -843,7 +1066,7 @@ void setPath() {
 }
 
 void SetPageDimensions() {
-  string paperType=getSetting<mem::string>("papertype");
+  string paperType=getSetting<string>("papertype");
 
   if(paperType == "" &&
      getSetting<double>("paperwidth") != 0.0 &&
@@ -862,46 +1085,60 @@ void SetPageDimensions() {
     if(paperType != "a4") {
       cerr << "Unknown paper size \'" << paperType << "\'; assuming a4." 
 	   << endl;
-      Setting("papertype")=mem::string("a4");
+      Setting("papertype")=string("a4");
     }
   }
 }
 
-bool pdf(const mem::string& texengine) {
+bool pdf(const string& texengine) {
   return texengine == "pdflatex" || texengine == "pdftex";
 }
 
-bool latex(const mem::string& texengine) {
+bool latex(const string& texengine) {
   return texengine == "latex" || texengine == "pdflatex";
 }
 
 string nativeformat() {
-  return pdf(getSetting<mem::string>("tex")) ? "pdf" : "eps";
+  return pdf(getSetting<string>("tex")) ? "pdf" : "eps";
+}
+
+string defaultformat() {
+  string format=getSetting<string>("outformat");
+  return (format == "") ? nativeformat() : format;
 }
 
 // TeX special command to set up currentmatrix for typesetting labels.
-const char *beginlabel(const mem::string& texengine) {
+const char *beginlabel(const string& texengine) {
   if(pdf(texengine))
-    return "\\special{pdf: q #5 0 0 cm}\\wd\\ASYbox 0pt\\dp\\ASYbox 0pt\\ht\\ASYbox 0pt";
-      return "\\special{ps: gsave currentpoint currentpoint translate [#5 0 0] concat neg exch neg exch translate}";
+    return "\\special{pdf: q #5 0 0 cm}"
+      "\\wd\\ASYbox 0pt\\dp\\ASYbox 0pt\\ht\\ASYbox 0pt";
+  else 
+    return "\\special{ps: gsave currentpoint currentpoint translate [#5 0 0] "
+      "concat neg exch neg exch translate}";
 }
 
 // TeX special command to restore currentmatrix after typesetting labels.
-const char *endlabel(const mem::string& texengine) {
+const char *endlabel(const string& texengine) {
   if(pdf(texengine))
     return "\\special{pdf: Q}";
-  return "\\special{ps: currentpoint grestore moveto}";
+  else
+    return "\\special{ps: currentpoint grestore moveto}";
 }
 
-// TeX special command to clip a label
-const char *clip(const mem::string& texengine) {
+// TeX macro to typeset raw postscript code
+const char *rawpostscript(const string& texengine) {
   if(pdf(texengine))
-    return "\\special{pdf: #1}";
-  return "\\special{ps: currentpoint currentpoint translate matrix currentmatrix\n[matrix defaultmatrix 0 get 0 0 matrix defaultmatrix 3 get\nmatrix currentmatrix 4 get matrix currentmatrix 5 get] setmatrix\n#1\nsetmatrix neg exch neg exch translate}";
+    return "\\def\\ASYraw#1{#1}";
+  else
+    return "\\def\\ASYraw#1{\n"
+      "currentpoint currentpoint translate matrix currentmatrix\n"
+      "100 12 div -100 12 div scale\n"
+      "#1\n"
+      "setmatrix neg exch neg exch translate}";
 }
 
 // Begin TeX special command.
-const char *beginspecial(const mem::string& texengine) {
+const char *beginspecial(const string& texengine) {
   if(pdf(texengine))
     return "\\special{pdf: ";
   return "\\special{ps: ";
@@ -912,10 +1149,26 @@ const char *endspecial() {
   return "}%";
 }
 
-mem::string texengine() {
-  mem::string path=getSetting<mem::string>("texpath");
-  mem::string tex=getSetting<mem::string>("tex");
-  return (path == "") ? tex : (mem::string) (path+"/"+tex);
+bool fataltex[]={false,true};
+const char *pdftexerrors[]={"! "," ==> Fatal error",NULL};
+const char *texerrors[]={"! ",NULL};
+
+// Messages that signify a TeX error.
+const char **texabort(const string& texengine)
+{
+  return settings::pdf(texengine) ? pdftexerrors : texerrors;
+}
+
+string texengine() 
+{
+  string command=getSetting<string>("texcommand");
+  return command.empty() ? getSetting<string>("tex") : command;
+}
+  
+string texprogram()
+{
+  string path=getSetting<string>("texpath");
+  return (path == "") ? texengine() : (string) (path+"/"+texengine());
 }
 
 int getScroll() 
@@ -937,9 +1190,6 @@ void setOptions(int argc, char *argv[])
 {
   argv0=argv[0];
 
-  if(setlocale (LC_ALL, "") == NULL)
-    perror("setlocale");
-  
   cout.precision(DBL_DIG);
   
   // Make configuration and history directory
@@ -954,7 +1204,7 @@ void setOptions(int argc, char *argv[])
   
   // Read user configuration file.
   setPath();
-  doConfig(getSetting<mem::string>("config"));
+  doConfig(getSetting<string>("config"));
   
   // Remember any changes to the defaultpen.
   initialdefaultpen=new camp::pen(defaultpen);
@@ -966,7 +1216,10 @@ void setOptions(int argc, char *argv[])
   if(!getSetting<bool>("debug")) GC_set_warn_proc(no_GCwarn);
 #endif  
 
-  // Set variables for the normal arguments.
+  if(setlocale (LC_ALL, "") == NULL && getSetting<bool>("debug"))
+    perror("setlocale");
+  
+  // Set variables for the file arguments.
   argCount = argc - optind;
   argList = argv + optind;
 
@@ -975,7 +1228,7 @@ void setOptions(int argc, char *argv[])
   
   if(getSetting<double>("paperwidth") != 0.0 && 
      getSetting<double>("paperheight") != 0.0)
-    Setting("papertype")=mem::string("");
+    Setting("papertype")=string("");
   
   SetPageDimensions();
   
