@@ -8,15 +8,12 @@
 #ifndef SETTINGS_H
 #define SETTINGS_H
 
-#include <string>
-#include <list>
 #include <fstream>
 
+#include "common.h"
 #include "pair.h"
 #include "item.h"
 
-using std::string;
-
 namespace types {
   class record;
 }
@@ -28,7 +25,10 @@ extern const char BUGREPORT[];
 
 extern const string docdir;
   
-extern int safe;
+extern bool safe;
+  
+extern bool global();
+
 enum origin {CENTER,BOTTOM,TOP,ZERO};
 //extern int origin;
   
@@ -37,8 +37,7 @@ extern int ShipoutNumber;
 extern const string suffix;
 extern const string guisuffix;
   
-extern bool TeXinitialized; // Is LaTeX process initialized?
-extern mem::string historyname;
+extern string historyname;
   
 void SetPageDimensions();
 
@@ -53,11 +52,14 @@ inline T getSetting(string name)
 }
 
 extern int verbose;
-extern string gvOptionPrefix;
-
+extern bool gray;
+extern bool bw;  
+extern bool rgb;
+extern bool cmyk;
+  
 bool view();
 bool trap();
-mem::string outname();
+string outname();
 
 void setOptions(int argc, char *argv[]);
 
@@ -67,17 +69,22 @@ char *getArg(int n);
  
 int getScroll();
   
-bool pdf(const mem::string& texengine);
-bool latex(const mem::string& texengine);
+bool pdf(const string& texengine);
+bool latex(const string& texengine);
   
 string nativeformat();
+string defaultformat();
   
-const char *beginlabel(const mem::string& texengine);
-const char *endlabel(const mem::string& texengine);
-const char *clip(const mem::string& texengine);
-const char *beginspecial(const mem::string& texengine);
+const char *beginlabel(const string& texengine);
+const char *endlabel(const string& texengine);
+const char *rawpostscript(const string& texengine);
+const char *beginspecial(const string& texengine);
 const char *endspecial();
   
-mem::string texengine();
+extern bool fataltex[];
+const char **texabort(const string& texengine);
+  
+string texengine();
+string texprogram();
 }
 #endif
