@@ -11,18 +11,18 @@
 
 #include <iostream>
 #include <cassert>
-#include <string>
-#include <map>
 
-#include "memory.h"
+#include "common.h"
 
-//using std::assert;
 using std::ostream;
-using std::string;
 
 namespace sym {
 
 struct GCInit {
+#ifdef _AIX
+  typedef char * GC_PTR;
+#endif
+  
   GCInit() {
 #ifdef USEGC
   GC_free_space_divisor = 2;
@@ -35,40 +35,40 @@ struct GCInit {
 struct symbol {
   static GCInit initialize;
 private:
-  std::string name;
+  string name;
 
 public:
-  static mem::map<CONST mem::string,symbol> dict;
+  static mem::map<CONST string,symbol> dict;
 
-  static symbol *specialTrans(std::string s) {
+  static symbol *specialTrans(string s) {
     assert(dict.find(s) == dict.end());
     return &(dict[s]=symbol(s,true));
   }
 
   symbol() : special(false) {}
-  symbol(std::string name, bool special=false)
+  symbol(string name, bool special=false)
     : name(name), special(special) {}
 
 public:
-  friend class mem::map<CONST mem::string,symbol>;
+  friend class mem::map<CONST string,symbol>;
   bool special; // NOTE: make this const (later).
   
   static symbol *initsym;
   static symbol *castsym;
   static symbol *ecastsym;
   
-  static symbol *literalTrans(std::string s) {
+  static symbol *literalTrans(string s) {
     if (dict.find(s) != dict.end())
       return &dict[s];
     else
       return &(dict[s]=symbol(s));
   }
 
-  static symbol *opTrans(std::string s) {
+  static symbol *opTrans(string s) {
     return literalTrans("operator "+s);
   }
 
-  static symbol *trans(std::string s) {
+  static symbol *trans(string s) {
     // Figure out whether it's an operator or an identifier by looking at the
     // first character.
     char c=s[0];
