@@ -13,17 +13,20 @@
 
 using std::ofstream;
 using settings::getSetting;
+
   
 namespace camp {
 
-std::list<string> TeXpipepreamble, TeXpreamble;
+mem::list<string> TeXpipepreamble, TeXpreamble;
   
 texfile::texfile(const string& texname, const bbox& box) : box(box)
 {
-  texengine=getSetting<mem::string>("tex");
+  texengine=getSetting<string>("tex");
+  inlinetex=getSetting<bool>("inlinetex");
+  Hoffset=inlinetex ? box.right : box.left;
   out=new ofstream(texname.c_str());
   if(!out || !*out) {
-    std::cerr << "Cannot write to " << texname << std::endl;
+    cerr << "Cannot write to " << texname << endl;
     throw handled_error();
   }
   out->setf(std::ios::fixed);
@@ -42,22 +45,41 @@ texfile::~texfile()
   
 void texfile::prologue()
 {
-  texdefines(*out);
+  if(inlinetex) {
+    std::ofstream *outpreamble=
+      new std::ofstream(auxname(getSetting<string>("outname"),"pre").c_str(),
+			std::ios::app);
+    texpreamble(*outpreamble,TeXpreamble);
+    outpreamble->close();
+  }
+  
+  texdefines(*out,TeXpreamble,false);
+  double width=box.right-box.left;
+  double height=box.top-box.bottom;
+  if(settings::pdf(texengine) && !inlinetex) {
+    double voffset=0.0;
+    if(settings::latex(texengine)) {
+      if(height < 12.0) voffset=height-12.0;
+    } else if(height < 10.0) voffset=height-10.0;
+
+    *out << "\\pdfhorigin=0bp" << newl
+	 << "\\pdfvorigin=" << voffset << "bp" << newl;
+    if(width > 0) 
+      *out << "\\pdfpagewidth=" << width << "bp" << newl;
+    if(height > 0)
+      *out << "\\pdfpageheight=" << height << "bp" << newl;
+  }
   if(settings::latex(texengine)) {
-    if(getSetting<bool>("inlinetex"))
+    if(inlinetex)
       *out << "\\setlength{\\unitlength}{1pt}" << newl;
     else {
       *out << "\\pagestyle{empty}" << newl
-	   << "\\textheight=2048pt" << newl
-	   << "\\textwidth=\\textheight" << newl;
+	   << "\\textheight=" << height+18.0 << "bp" << newl
+	   << "\\textwidth=" << width+18.0 << "bp" << newl;
       if(settings::pdf(texengine))
 	*out << "\\oddsidemargin=-17.61pt" << newl
 	     << "\\evensidemargin=\\oddsidemargin" << newl
-	     << "\\topmargin=-37.01pt" << newl
-	     << "\\pdfhorigin=0bp" << newl
-	     << "\\pdfvorigin=0bp" << newl
-	     << "\\pdfpagewidth=" << box.right-box.left << "bp" << newl
-	     << "\\pdfpageheight=" << box.top-box.bottom << "bp" << newl;
+	     << "\\topmargin=-37.01pt" << newl;
       *out << "\\begin{document}" << newl;
     }
   } else {
@@ -65,11 +87,7 @@ void texfile::prologue()
 	 << "\\voffset=54.0pt" << newl;
     if(settings::pdf(texengine)) {
       *out << "\\hoffset=-20pt" << newl
-	   << "\\voffset=0pt" << newl
-	   << "\\pdfhorigin=0bp" << newl
-	   << "\\pdfvorigin=0bp" << newl
-	   << "\\pdfpagewidth=" << box.right-box.left << "bp" << newl
-	   << "\\pdfpageheight=" << box.top-box.bottom << "bp" << newl;
+	   << "\\voffset=0pt" << newl;
     }
   }
 }
@@ -82,15 +100,22 @@ void texfile::beginlayer(const string& psname)
       *out << "[bb=" << box.left << " " << box.bottom << " "
 	   << box.right << " " << box.top << "]";
     *out << "{" << psname << "}%" << newl;
+    if(!inlinetex)
+      *out << "\\kern-" << (box.right-box.left)*ps2tex << "pt%" << newl;
   }
 }
 
 void texfile::endlayer()
 {
-  if(box.right > box.left && box.top > box.bottom)
+  if(inlinetex && (box.right > box.left && box.top > box.bottom))
     *out << "\\kern-" << (box.right-box.left)*ps2tex << "pt%" << newl;
 }
 
+void texfile::writeshifted(path p, bool newPath)
+{
+  write(p.transformed(shift(pair(-Hoffset,-box.bottom))),newPath);
+}
+
 void texfile::setlatexcolor(pen p)
 {
   if(p.cmyk() && (!lastpen.cmyk() || 
@@ -152,13 +177,23 @@ void texfile::grestore()
   psfile::grestore(true);
   *out << settings::endspecial() << newl;
 }
-  
-void texfile::openclip() 
+
+void texfile::beginspecial() 
 {
-  *out << "\\ASYput{" << newl;
+  *out << settings::beginspecial(texengine);
 }
   
-void texfile::closeclip() 
+void texfile::endspecial() 
+{
+  *out << settings::endspecial() << newl;
+}
+  
+void texfile::beginraw() 
+{
+  *out << "\\ASYraw{" << newl;
+}
+  
+void texfile::endraw() 
 {
   *out << "}%" << newl;
 }
@@ -171,7 +206,7 @@ void texfile::put(const string& label, const transform& T, const pair& z,
   if(label.empty()) return;
   
   *out << "\\ASYalign"
-       << "(" << (z.getx()-box.right)*ps2tex
+       << "(" << (z.getx()-Hoffset)*ps2tex
        << "," << (z.gety()-box.bottom)*ps2tex
        << ")(" << align.getx()
        << "," << align.gety() 
@@ -183,7 +218,7 @@ void texfile::put(const string& label, const transform& T, const pair& z,
 void texfile::epilogue()
 {
   if(settings::latex(texengine)) {
-    if(!getSetting<bool>("inlinetex"))
+    if(!inlinetex)
       *out << "\\end{document}" << newl;
   } else {
       *out << "\\bye" << newl;
