@@ -9,11 +9,10 @@
 #define TEXFILE_H
 
 #include <fstream>
-#include <string>
 #include <iomanip>
 #include <iostream>
-#include <list>
 
+#include "common.h"
 #include "pair.h"
 #include "bbox.h"
 #include "pen.h"
@@ -24,11 +23,9 @@
 #include "psfile.h"
 #include "settings.h"
 
-using std::string;
-
 namespace camp {
 
-extern std::list<string> TeXpipepreamble, TeXpreamble;
+extern mem::list<string> TeXpipepreamble, TeXpreamble;
 
 const double tex2ps=72.0/72.27;
 const double ps2tex=1.0/tex2ps;
@@ -36,24 +33,26 @@ const double ps2tex=1.0/tex2ps;
 template<class T>
 void texdocumentclass(T& out, bool pipe=false)
 {
-  bool latex=settings::latex(settings::getSetting<mem::string>("tex"));
-  if(latex && (pipe || !settings::getSetting<bool>("inlinetex")))
+  if(settings::latex(settings::getSetting<string>("tex")) &&
+     (pipe || !settings::getSetting<bool>("inlinetex")))
     out << "\\documentclass[12pt]{article}" << newl;
 }
   
 template<class T>
-void texpreamble(T& out, std::list<string>& preamble=TeXpreamble)
+void texpreamble(T& out, mem::list<string>& preamble=TeXpreamble)
 {
-  for(std::list<string>::iterator p=preamble.begin(); p != preamble.end(); ++p)
+  for(mem::list<string>::iterator p=preamble.begin();
+      p != preamble.end(); ++p)
     out << stripblanklines(*p);
 }
 
 template<class T>
-void texdefines(T& out, std::list<string>& preamble=TeXpreamble,
+void texdefines(T& out, mem::list<string>& preamble=TeXpreamble,
 		bool pipe=false)
 {
-  mem::string texengine=settings::getSetting<mem::string>("tex");
-  texpreamble(out,preamble);
+  string texengine=settings::getSetting<string>("tex");
+  if(pipe || !settings::getSetting<bool>("inlinetex"))
+    texpreamble(out,preamble);
   out << "\\newbox\\ASYbox" << newl
       << "\\newdimen\\ASYdimen" << newl
       << "\\def\\ASYbase#1#2{\\setbox\\ASYbox=\\hbox{#1}"
@@ -69,9 +68,19 @@ void texdefines(T& out, std::list<string>& preamble=TeXpreamble,
       << "\\box\\ASYbox%" << newl
       << settings::endlabel(texengine) << "%" << newl
       << "}}" << newl
-      << "\\def\\ASYput#1{%" << newl
-      << settings::clip(texengine) << "}" << newl;
+      << settings::rawpostscript(texengine) << newl;
   
+  if(pipe) {
+    // Make tex pipe aware of a previously generated aux file.
+    string name=auxname(settings::outname(),"aux");
+    std::ifstream fin(name.c_str());
+    if(fin) {
+      std::ofstream fout("texput.aux");
+      string s;
+      while(getline(fin,s))
+	fout << s << endl;
+    }
+  }
   if(settings::latex(texengine)) {
     if(pipe || !settings::getSetting<bool>("inlinetex")) {
       out << "\\usepackage{graphicx}" << newl;
@@ -88,7 +97,9 @@ void texdefines(T& out, std::list<string>& preamble=TeXpreamble,
   
 class texfile : public psfile {
   bbox box;
-  mem::string texengine;
+  string texengine;
+  bool inlinetex;
+  double Hoffset;
 
 public:
   texfile(const string& texname, const bbox& box);
@@ -105,14 +116,20 @@ public:
   
   void grestore();
   
-  void openclip();
+  void beginspecial();
   
-  void closeclip();
+  void endspecial();
   
-  void writeshifted(path p, bool newPath=true) {
-    write(p.transformed(shift(pair(-box.right,-box.bottom))),newPath);
+  void beginraw();
+  
+  void endraw();
+  
+  void writepair(pair z) {
+    *out << z;
   }
   
+  void writeshifted(path p, bool newPath=true);
+  
   // Draws label transformed by T at position z.
   void put(const string& label, const transform& T, const pair& z,
 	   const pair& Align);
@@ -120,10 +137,6 @@ public:
   void beginlayer(const string& psname);
   void endlayer();
   
-  void verbatim(const string& s) {
-    *out << s << newl;
-  }
-  
 };
 
 } //namespace camp
