@@ -14,46 +14,20 @@
 #include "runtime.h"
 #include "access.h"
 
-using std::cout;
-
 namespace types {
 
 /* Base types */
-primitiveTy pVoid(ty_void);
-ty *primVoid() { return &pVoid; }
+#define PRIMITIVE(name,Name,asyName) \
+  primitiveTy p##Name(ty_##name); \
+  ty *prim##Name() { return &p##Name; }
+#define PRIMERROR
+#include <primitives.h>
+#undef PRIMERROR
+#undef PRIMITIVE
+                             
 nullTy pNull;
 ty *primNull() { return &pNull; }
-primitiveTy pError(ty_error);
-ty *primError() { return &pError; }
-primitiveTy pBoolean(ty_boolean);
-ty *primBoolean() { return &pBoolean; }
-primitiveTy pInt(ty_int);
-ty *primInt() { return &pInt; }
-primitiveTy pReal(ty_real);
-ty *primReal() { return &pReal; }
-primitiveTy pString(ty_string);
-ty *primString() { return &pString; }
-primitiveTy pPair(ty_pair);
-ty *primPair() { return &pPair; }
-primitiveTy pTriple(ty_triple);
-ty *primTriple() { return &pTriple; }
-primitiveTy pTransform(ty_transform);
-ty *primTransform() { return &pTransform; }
-primitiveTy pGuide(ty_guide);
-ty *primGuide() { return &pGuide; }
-primitiveTy pPath(ty_path);
-ty *primPath() { return &pPath; }
-primitiveTy pPen(ty_pen);
-ty *primPen() { return &pPen; }
-primitiveTy pPicture(ty_picture);
-ty *primPicture() { return &pPicture; }
-primitiveTy pFile(ty_file);
-ty *primFile() { return &pFile; }
-primitiveTy pCode(ty_code);
-ty *primCode() { return &pCode; }
-primitiveTy primArray_(ty_array);
-ty *primArray() { return &primArray_; }
-  
+
 array boolArray_(primBoolean());
 ty *boolArray() { return &boolArray_; }
 array intArray_(primInt());
@@ -104,12 +78,15 @@ array stringArray3_(stringArray2());
 ty *stringArray3() { return &stringArray3_; }
   
 const char *names[] = {
-  "void", "null",
-  "<structure>", "<function>", "<error>", "<overloaded>",
-  "bool", "int", "real",
-  "string",
-  "pair", "triple", "transform", "guide", "path", "pen", "frame",
-  "file", "code",
+  "null",
+  "<structure>", "<function>", "<overloaded>",
+  
+#define PRIMITIVE(name,Name,asyName) #asyName,
+#define PRIMERROR
+#include <primitives.h>
+#undef PRIMERROR
+#undef PRIMITIVE
+
   "<array>"
 };
 
@@ -123,71 +100,46 @@ void ty::print(ostream& out) const
 
 trans::varEntry *primitiveTy::virtualField(symbol *id, signature *sig)
 {
+#define FIELD(Type, name, func) \
+  if (sig == 0 && id == symbol::trans(name)) { \
+    static trans::bltinAccess a(run::func); \
+    static trans::varEntry v(prim##Type(), &a, 0, position()); \
+    return &v; \
+  }
+
   switch (kind) {
     case ty_pair:
-      if (sig == 0 && id == symbol::trans("x")) {
-        static trans::bltinAccess a(run::pairXPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
-      if (sig == 0 && id == symbol::trans("y")) {
-        static trans::bltinAccess a(run::pairYPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
+      FIELD(Real,"x",pairXPart);
+      FIELD(Real,"y",pairYPart);
       break;
     case ty_triple:
-      if (sig == 0 && id == symbol::trans("x")) {
-        static trans::bltinAccess a(run::tripleXPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
-      if (sig == 0 && id == symbol::trans("y")) {
-        static trans::bltinAccess a(run::tripleYPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
-      if (sig == 0 && id == symbol::trans("z")) {
-        static trans::bltinAccess a(run::tripleZPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
+      FIELD(Real,"x",tripleXPart);
+      FIELD(Real,"y",tripleYPart);
+      FIELD(Real,"z",tripleZPart);
       break;
     case ty_transform:
-      if (sig == 0 && id == symbol::trans("x")) {
-        static trans::bltinAccess a(run::transformXPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
-      if (sig == 0 && id == symbol::trans("y")) {
-        static trans::bltinAccess a(run::transformYPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
-      if (sig == 0 && id == symbol::trans("xx")) {
-        static trans::bltinAccess a(run::transformXXPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
-      if (sig == 0 && id == symbol::trans("xy")) {
-        static trans::bltinAccess a(run::transformXYPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
-      if (sig == 0 && id == symbol::trans("yx")) {
-        static trans::bltinAccess a(run::transformYXPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
-      if (sig == 0 && id == symbol::trans("yy")) {
-        static trans::bltinAccess a(run::transformYYPart);
-        static trans::varEntry v(primReal(), &a, 0);
-        return &v;
-      }
+      FIELD(Real,"x",transformXPart);
+      FIELD(Real,"y",transformYPart);
+      FIELD(Real,"xx",transformXXPart);
+      FIELD(Real,"xy",transformXYPart);
+      FIELD(Real,"yx",transformYXPart);
+      FIELD(Real,"yy",transformYYPart);
+      break;
+    case ty_tensionSpecifier:
+      FIELD(Real,"out",tensionSpecifierOutPart);
+      FIELD(Real,"in",tensionSpecifierInPart);
+      FIELD(Boolean,"atLeast",tensionSpecifierAtleastPart);
+      break;
+    case ty_curlSpecifier:
+      FIELD(Real,"value",curlSpecifierValuePart);
+      FIELD(Int,"side",curlSpecifierSidePart);
+      break;
     default:
       break;
   }
   return 0;
+
+#undef FIELD
 }
 
 ty *ty::virtualFieldGetType(symbol *id)
@@ -282,20 +234,20 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   if (sig == 0 && id == symbol::trans("length"))
   {
     static trans::bltinAccess a(run::arrayLength);
-    static trans::varEntry v(primInt(), &a, 0);
+    static trans::varEntry v(primInt(), &a, 0, position());
     return &v;
   }
   if (sig == 0 && id == symbol::trans("cyclicflag"))
   {
     static trans::bltinAccess a(run::arrayCyclicFlag);
-    static trans::varEntry v(primBoolean(), &a, 0);
+    static trans::varEntry v(primBoolean(), &a, 0, position());
     return &v;
   }
   if (id == symbol::trans("cyclic") &&
       equivalent(sig, cyclicType()->getSignature()))
   {
     static trans::bltinAccess a(run::arrayCyclic);
-    static trans::varEntry v(cyclicType(), &a, 0);
+    static trans::varEntry v(cyclicType(), &a, 0, position());
     return &v;
   }
   if (id == symbol::trans("push") &&
@@ -303,7 +255,7 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   {
     static trans::bltinAccess a(run::arrayPush);
     // v needs to be dynamic, as the push type differs among arrays.
-    trans::varEntry *v = new trans::varEntry(pushType(), &a, 0);
+    trans::varEntry *v = new trans::varEntry(pushType(), &a, 0, position());
 
     return v;
   }
@@ -312,7 +264,7 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   {
     static trans::bltinAccess a(run::arrayPop);
     // v needs to be dynamic, as the pop type differs among arrays.
-    trans::varEntry *v = new trans::varEntry(popType(), &a, 0);
+    trans::varEntry *v = new trans::varEntry(popType(), &a, 0, position());
 
     return v;
   }
@@ -321,7 +273,7 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   {
     static trans::bltinAccess a(run::arrayAppend);
     // v needs to be dynamic, as the append type differs among arrays.
-    trans::varEntry *v = new trans::varEntry(appendType(), &a, 0);
+    trans::varEntry *v = new trans::varEntry(appendType(), &a, 0, position());
 
     return v;
   }
@@ -330,7 +282,7 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   {
     static trans::bltinAccess a(run::arrayInsert);
     // v needs to be dynamic, as the insert type differs among arrays.
-    trans::varEntry *v = new trans::varEntry(insertType(), &a, 0);
+    trans::varEntry *v = new trans::varEntry(insertType(), &a, 0, position());
 
     return v;
   }
@@ -339,7 +291,7 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
   {
     static trans::bltinAccess a(run::arrayDelete);
     // v needs to be dynamic, as the delete type differs among arrays.
-    trans::varEntry *v = new trans::varEntry(deleteType(), &a, 0);
+    trans::varEntry *v = new trans::varEntry(deleteType(), &a, 0, position());
 
     return v;
   }
@@ -420,28 +372,6 @@ size_t signature::hash() {
   return x;
 }
 
-#if 0
-int numFormalsMatch(signature *target, signature *source)
-{
-  assert(target && source);
-
-  int matches=0;
-
-  formal_vector::iterator t    =target->formals.begin(),
-                          t_end=target->formals.end(),
-                          s    =source->formals.begin(),
-                          s_end=source->formals.end();
-  for (; t!=t_end; ++t)
-    if (s!=s_end && castable(*t, *s)) {
-      if (equivalent(*t, *s))
-        ++matches;
-      ++s;
-    }
-
-  return matches;
-}
-#endif
-
 trans::access *function::initializer() {
   static trans::bltinAccess a(run::pushNullFunction);
   return &a;
@@ -505,117 +435,6 @@ bool overloaded::castable(ty *target, caster &c)
   return false;
 }
 
-#if 0 //{{{
-ty *overloaded::resolve(signature *key)
-{
-  cout << "sig: " << (key ? "fun " : "trivial\n");
-  cout << "candidates: " << sub.size();
-
-  overloaded set;
-  
-  // Pick out all applicable signatures.
-  for(ty_vector::iterator t = sub.begin();
-      t != sub.end();
-      ++t)
-  {
-    signature *nsig = (*t)->getSignature();
-   
-    if (castable(nsig, key)) {
-      set.add(*t);
-
-      // Shortcut for simple (ie. non-function) variables.
-      if (key == 0)
-	return (*t);
-    }
-  }
-
-  ty_vector& candidates = set.sub;
-  cout << " applicable: " << candidates.size() << endl; 
-  if (candidates.size() <= 1)
-    return set.simplify();
-
-  // Try to further resolve candidates by checking for number of
-  // arguments exactly matched.
-  for (int n = key->getNumFormals(); n > 0; n--)
-  {
-    ty_vector matches;
-    for (ty_vector::iterator p = candidates.begin();
-         p != candidates.end();
-	 ++p)
-    {
-      if (numFormalsMatch((*p)->getSignature(), key) >= n) {
-        matches.push_back(*p);
-      }
-    }
-
-    if (matches.size() == 1)
-      return matches.front();
-    if (matches.size() > 1)
-      break;
-  }
-
-  return new overloaded(set);
-}
-
-ty *overloaded::resolve(signature *key, symbol *name, position pos)
-{
-  cout << "name: ";
-  if (name)
-    cout << *name;
-  else
-    cout << "none";
-  cout << "position: " << pos;
-
-  cout << " sig: " << (key ? "fun " : "trivial\n");
-  cout << "candidates: " << sub.size();
-
-  overloaded set;
-  
-  // Pick out all applicable signatures.
-  for(ty_vector::iterator t = sub.begin();
-      t != sub.end();
-      ++t)
-  {
-    signature *nsig = (*t)->getSignature();
-   
-    if (castable(nsig, key)) {
-      set.add(*t);
-
-      // Shortcut for simple (ie. non-function) variables.
-      if (key == 0)
-	return (*t);
-    }
-  }
-
-  ty_vector& candidates = set.sub;
-  cout << " applicable: " << candidates.size() << endl; 
-  if (candidates.size() <= 1)
-    return set.simplify();
-
-  // Try to further resolve candidates by checking for number of
-  // arguments exactly matched.
-  for (int n = key->getNumFormals(); n > 0; n--)
-  {
-    ty_vector matches;
-    for (ty_vector::iterator p = candidates.begin();
-         p != candidates.end();
-	 ++p)
-    {
-      if (numFormalsMatch((*p)->getSignature(), key) >= n) {
-        matches.push_back(*p);
-      }
-    }
-
-    if (matches.size() == 1)
-      return matches.front();
-    if (matches.size() > 1)
-      break;
-  }
-
-  return new overloaded(set);
-}
-#endif //}}}
-
 bool equivalent(ty *t1, ty *t2)
 {
   // The same pointer must point to the same type.
