@@ -27,56 +27,8 @@
 
 using namespace settings;
 
-#ifdef __CYGWIN__
-#define CERR cout
-#else
-#define CERR cerr
-#endif  
-
 bool False=false;
 
-string stripExt(const string& name, const string& ext)
-{
-  string suffix = "." + ext;
-  size_t p=name.rfind(suffix);
-  if (p == name.length()-suffix.length())
-    return name.substr(0,p);
-  else return name;
-}
-
-void backslashToSlash(string& s) 
-{
-  size_t p;
-  while ((p=s.find('\\')) < string::npos)
-    s[p]='/';
-}
-
-void spaceToUnderscore(string& s) 
-{
-  size_t p;
-  while ((p=s.find(' ')) < string::npos)
-    s[p]='_';
-}
-
-#ifdef __CYGWIN__
-string Getenv(const char *name, bool quote)
-{
-  char *s=getenv(name);
-  if(!s) return "";
-  string S=string(s);
-  backslashToSlash(S);
-  if(quote) S="'"+S+"'";
-  return S;
-}
-#else
-string Getenv(const char *name, bool)
-{
-  char *s=getenv(name);
-  if(!s) return "";
-  return string(s);
-}
-#endif
-
 string stripDir(string name)
 {
   size_t p;
@@ -89,11 +41,71 @@ string stripDir(string name)
   return name;
 }
 
+string stripFile(string name)
+{
+  size_t p;
+#ifdef __CYGWIN__  
+  p=name.rfind('\\');
+  if(p < string::npos) name.erase(p+1);
+#endif  
+  p=name.rfind('/');
+  if(p < string::npos) name.erase(p+1);
+  return name;
+}
+  
+string stripExt(string name, const string& ext)
+{
+  string suffix="."+ext;
+  size_t p=name.rfind(suffix);
+  size_t n=suffix.length();
+  if(n == 1 || p == name.length()-n)
+    return name.substr(0,p);
+  else return name;
+}
+
+void backslashToSlash(string& s) 
+{
+  size_t p;
+  while((p=s.find('\\')) < string::npos)
+    s[p]='/';
+}
+
+void spaceToUnderscore(string& s) 
+{
+  size_t p;
+  while((p=s.find(' ')) < string::npos)
+    s[p]='_';
+}
+
+string Getenv(const char *name, bool msdos)
+{
+  char *s=getenv(name);
+  if(!s) return "";
+  string S=string(s);
+  if(msdos) backslashToSlash(S);
+  return S;
+}
+
+void writeDisabled()
+{
+  camp::reportError("Write/cd to other directories disabled; override with option -global");
+}
+
+void checkLocal(string name)
+{
+  if(global()) return;
+#ifdef __CYGWIN__  
+  if(name.rfind('\\') < string::npos) writeDisabled();
+#endif  
+  if(name.rfind('/') < string::npos) writeDisabled();
+  return;
+}
+
 string buildname(string name, string suffix, string aux, bool stripdir) 
 {
-  if(stripdir) stripDir(name);
+  if(stripdir) name=stripDir(name);
     
-  name = stripExt(name,getSetting<mem::string>("outformat"));
+  name=stripExt(name,defaultformat());
   name += aux;
   if(!suffix.empty()) name += "."+suffix;
   return name;
@@ -104,16 +116,6 @@ string auxname(string filename, string suffix)
   return buildname(filename,suffix,"_");
 }
   
-bool checkFormatString(const string& format)
-{
-  if(format.find(' ') != string::npos) { // Avoid potential security hole
-    ostringstream msg;
-    msg << "output format \'" << format << "\' is invalid";
-    camp::reportError(msg);
-  }
-  return true;
-}
-  
 // Return an argv array corresponding to the fields in command delimited
 // by spaces not within matching single quotes.
 char **args(const char *command)
@@ -164,12 +166,12 @@ char **args(const char *command)
 
 void execError(const char *command, const char *hint, const char *application)
 {
-    CERR << "Cannot execute " << command << endl;
+    cerr << "Cannot execute " << command << endl;
     if(application == "") application=hint;
-    string s=string(hint);
-    transform(s.begin(), s.end(), s.begin(), toupper);        
-    if(hint) 
-      CERR << "Please put in " << getSetting<mem::string>("config")
+    if(hint) {
+      string s=string(hint);
+      transform(s.begin(), s.end(), s.begin(), toupper);        
+      cerr << "Please put in " << getSetting<string>("config")
 	   << ": " << endl << endl
 	   << "import settings;" << endl
            << hint << "=\"PATH\";" << endl << endl
@@ -178,6 +180,7 @@ void execError(const char *command, const char *hint, const char *application)
 	   << "Alternatively, set the environment variable ASYMPTOTE_" << s 
 	   << endl << "or use the command line option -" << hint 
 	   << "=\"PATH\"" << endl;
+    }
     exit(-1);
 }
 						    
@@ -254,29 +257,25 @@ int System(const ostringstream& command, int quiet, bool wait,
   return System(command.str().c_str(),quiet,wait,hint,application,pid);
 }
 
-string stripblanklines(string& s)
+string stripblanklines(const string& s)
 {
+  string S=string(s);
   bool blank=true;
-  const char *t=s.c_str();
-  size_t len=s.length();
+  const char *t=S.c_str();
+  size_t len=S.length();
   
   for(size_t i=0; i < len; i++) {
     if(t[i] == '\n') {
-      if(blank) s[i]=' ';
+      if(blank) S[i]=' ';
       else blank=true;
     } else if(t[i] != '\t' && t[i] != ' ') blank=false;
   }
-  return s;
+  return S;
 }
 
-static char *startpath=NULL;
+char *startpath=NULL;
 char *currentpath=NULL;
 
-char *startPath()
-{
-  return startpath;
-}
-
 void noPath()
 {
   camp::reportError("Cannot get current path");
@@ -297,12 +296,24 @@ char *getPath(char *p)
   return p;
 }
 
-int setPath(const char *s)
+const char *setPath(const char *s, bool quiet)
 {
-  if(s != NULL && *s != 0) {
-    if(startpath == NULL) startpath=getPath(startpath);
-    return chdir(s);
-  } return 0;
+  if(startpath == NULL) startpath=getPath(startpath);
+  if(s == NULL || *s == 0) s=startpath;
+  int rc=chdir(s);
+  if(rc != 0) {
+    ostringstream buf;
+    buf << "Cannot change to directory '" << s << "'";
+    camp::reportError(buf);
+  }
+  char *p=getPath();
+  if(p) {
+    if(interact::interactive && !quiet)
+      cout << p << endl;
+    else if(verbose > 1)
+      cout << "cd " << p << endl;
+  }
+  return p;
 }
 
 void popupHelp() {
@@ -316,7 +327,7 @@ void popupHelp() {
   // viewer again.
   if (pid==0 || (waitpid(pid, &status, WNOHANG) == pid)) {
     ostringstream cmd;
-    cmd << "'" << getSetting<mem::string>("pdfviewer") << "' " 
+    cmd << "'" << getSetting<string>("pdfviewer") << "' " 
       << docdir << "/asymptote.pdf";
     status=System(cmd,0,false,"pdfviewer","your PDF viewer",&pid);
   }
